# KCR analyser Installation Guide

The KCR analyser is a model-based safety analysis tool for static
systems described in the KCR language. It provides automatic
computation of reliability and minimal cutsets from a KCR model of
the system using SMT-based and BDD-based techniques.

The analyser is a self-contained jar archive which can be downloaded [here](http://www.onera.fr/en/staff/kevin-delmas?page=1).

The only dependencies of the KCR analyzer are:
+ The Java Runtime Environmnent version 8 [JRE 1.8](http://www.oracle.com/technetwork/java/javase/downloads/jre8-downloads-2133155.html) or newer.
+ Microsoft's Z3 SMT-solver version 4.4.1 [Z3 4.4.1][2]

[1]: #run-KCR-analyser
[2]: #install-Z3

## Installing dependencies

### Java 8

You need a working installation of the Java Runtime Environment
version 8 (either OpenJDK or Oracle will do).  Installation procedures
may vary depending on your system (Windows, OSX, Linux), please follow
the official guidelines for your system.


### Microsoft Z3 with Java Bindings

The easyiest way to install Z3 is to download and unpack one of the
precompiled releases available [here](https://github.com/Z3Prover/z3/releases) for you system version.

If a precompiled version is not available for your system, Z3 can
be built from sources by following instructions available [on github](https://github.com/Z3Prover/z3).

If compiling from sources, do not forget to add the ``--java`` switch
to the command-line to enable compilation of the java bindings:

``python scripts/mk_make.py --java``

Optionally, you can use the ``--prefix="path"`` swith to change the default installation path of Z3 used when doing ``make install``.

In your shell configuration file, let us name ``Z3_HOME`` the folder in which Z3 has been
installed (either by decompressing the precompiled release or by
installing from sources).

``export Z3_HOME="path where Z3 has been installed"``

For the KCR analyser to work properly with Z3, the dynamic libraries
``libz3.{dylib|dll|so}`` and ``libz3java.{dylib|dll|so}`` located in
the ``${Z3_HOME}/bin`` directory of the Z3 installation folder must be
added to the dynamic library path of your system as follows:

- For Linux and MacOS:

`` export LD_LIBRARY_PATH="${Z3_HOME}/bin":${LD_LIBRARY_PATH} ``

- for Windows

`` set PATH=%Z3_HOME%/bin;%PATH%" ``


## Installing and using the KCR analyser

There is no installation procedure for the KCR analyzer itself, simply
download the self-contained jar somewhere on your filesystem, it is
ready to use (provided all dependencies are installed).

Nevertheless an optional install script executed as follow ``./install.sh -p <PATH>``  is available. 
It creates the KCR directory in ``PATH`` and adds the jar archive, execution script and uninstall script. The tool can then be simply called by ``kcr-analyser`` command instead of the basic ``java -jar kcr-analyser.jar `` command.

### Installing the emacs mode ``kcr.el``

Let name ``KCR_HOME`` the folder where you unzip the archive containing the tool.
The location of the files ``KCR_HOME/scripts/kcr-mode.el `` and ``KCR_HOME/scripts/kcr-mode-autoload.el ``
must be added to your emacs init file which must be the ``.emacs`` file located in your home directory.
Add the following lines to your ``.emacs`` file.

``(load "KCR_HOME/scripts/kcr-mode")``
 `` (load "KCR_HOME/scripts/kcr-mode-autoloads")``

 These files will automatically run the emacs ``kcr-mode`` when a ``.kcr`` file is
 opened.

### Running the KCR analyser

The KCR analyser is run as follows 

`` java -Xmx1024m -jar tool_2.11-0.2-one-jar.jar [options] <KCR files> ``

 where ``KCR
files`` are the input files describing the system and the analysis
configuration.  The tool exits if none specified or files do not
exist, and the ``-Xmx`` switch is used to allocate RAM space to the
JVM, and can be adapted depending on the quantity of RAM available on
your system.


The (non exhaustive) list of command line options is

  ``--help`` prints this usage text

  ``-c <value> | --configuration <value>``
        the configuration identifier to analyse. If not defined the analyser only
        parse and type check input files

  ``--compute-r``
        asks to compute the system reliability

  ``--print-phi-dot <value>``
        exports characteristic function as dot graph

  ``--check-r``
        checks the system reliability according to requirement

  ``--compute-min-card``
        asks to compute minimal cardinality of MCS

  ``--check-min-card``
        checks the minimal cardnality of MCS according to requirement

  ``--compute-MCS <value>``
        enumerates k-size MCS where 0 means all cutsets
        
   ``-o | --optmise ``
        run the exploration problem on the selected file and the selected configuration
        
   ``--gui`` 
        run in gui mode, special editor / runner to edit, analyse and export KCR files
      
   ``--run-cut-perf``
       generate cutset enumeration execution time table for bench in bench/cutsetEnumeration/KCR
       
   ``--run-cut-perf``
        generate exploration execution time table for bench in bench/exploration/KCR
        
   ``--run-stmdd-perf``
             generate STMDD vs BDD time table for bench in bench/exploration/KCR
             
          
       
        

### KCR examples

Various benchmark systems for cutsets enumeration and architecture optimisation are given respectively in ``bench/cutEnumeration/KCR`` and ``bench/exploration/KCR``.

The model of benchmarks can by translated for the tools [XSAP](https://xsap.fbk.eu/), [XFTA](http://www.altarica-association.org/) and [HipHOPS](http://hip-hops.eu/) thanks
to commands respectively ``--export-smv``, ``--export-xfta`` and ``--export-hiphops`` in command line mode or in the option menu in gui mode.

### KCR and external tools

The cutset generation and architecture optimisation can be performed by external tools in gui mode.
But the path to this tool must be provided either in your ``PATH`` variable or given in the path menu
of the gui.