package analyzers

import com.microsoft.z3.{BoolExpr => Z3BoolExpr, Expr => Z3Expr, Status => Z3Status}
/**
  * Trait checking if the order requirement is met by the initial architecture
  */
trait AcceptableCardAnalyzer extends AcceptabilityAnalyzer {
  self: Parsing with SMTTranslation =>

  lazy val cardOK: Option[Boolean] = isAcceptableCard

  /**
    * check if it exists a valuation of system events containing at most minCardValue true events
    * such that the structure function phi is true
    *
    * @return if the problem is UNSAT or not
    */
  private def isAcceptableCard: Option[Boolean] = {
    for (obj <- conf.minCardValue) yield {
      val s = _SMTContext.mkSolver()
      val eventsList = phi.events.toList
      val consts = eventsList.map(e => (e._2, _SMTContext.mkBoolConst(e._1.toString()))).unzip
      s.add(phi.expr.substitute(consts._1.toArray[Z3Expr], consts._2.toArray[Z3Expr]).asInstanceOf[Z3BoolExpr])
      s.add(_SMTContext.atMostN(consts._2, obj.repr.toInt - 1))
      val res = s.check() == Z3Status.UNSATISFIABLE
      s.dispose()
      res
    }
  }
}
