package analyzers

import java.io.File

import ast.dsl.{BoolIdent, CompInstanceIdent, InstBoolIdent, StdDSLSymbol}
import translators.DSLToHipHOPSTranslator
import util.analyzers._
import util.io.{FileManager,SystemInfo}

import scala.io.Source
import scala.language.postfixOps
import scala.sys.process.Process
import scala.xml.XML
import scala.xml.dtd.DocType

/**
  * Created by kdelmas on 27/09/17.
  */

/**
  * Base trait for cutsets computation
  */
trait CutEnumerator extends ConfigurationAnalyzer {
  self: Parsing =>

  /**
    * Compute cutsets up to an user defined cutset size
    *
    * @param max maximal cutset size if defined, no bound otherwise
    * @return the set of minimal cutsets
    */
  def enumerateCutUntil(max: Option[Int]): Iterable[Set[InstBoolIdent]]

  /**
    * Profiler for timing the cutsets computation
    */
  val profilerCutEnumeration: Profiler[Option[Int], Iterable[Set[InstBoolIdent]]]
}