package analyzers

import ast.dsl.InstBoolIdent
import util.analyzers._

/**
  * Cutsets computation with BDD method
  */
trait CutEnumeratorBDD extends CutEnumerator with BDDFactoryField {
  self: Parsing with SMTTranslation =>
  private lazy val cutPhiBDD = new Phi(phi) with PhiCutEnumerationBDD {
    val BDDFactory: BDDFactory with Z3BoolExprToBDDTranslator = self.BDDFactory
  }

  /**
    * Compute minimal cutsets with BDD method
    *
    * @param max maximal cutset size if defined, no bound otherwise
    * @return the set of minimal cutsets
    */
  def enumerateCutUntil(max: Option[Int]): Iterable[Set[InstBoolIdent]] = {
    max match {
      case Some(m: Int) => cutPhiBDD.computeCutsUntil(m)
      case None => cutPhiBDD.computeAllCuts()
    }
  }

  override val profilerCutEnumeration = new Profiler[Option[Int], Iterable[Set[InstBoolIdent]]] {
    def runWithProfile(i: Option[Int])(iterations: Int = 1): (Iterable[Set[InstBoolIdent]], ExecutionTimes) = {
      cutPhiBDD.phiCutEnumerationProfiler.runWithProfile(i)(iterations)
    }
  }

}
