package analyzers

import ast.dsl.InstBoolIdent
import util.analyzers.{ExecutionTimes, Phi, PhiCutEnumerationPB, Profiler}

/**
  * Cutsets computation with Pseudo-Boolean method
  */
trait CutEnumeratorPB extends CutEnumerator {
  self: Parsing with SMTTranslation =>
  private lazy val cutPhiPB = new Phi(phi) with PhiCutEnumerationPB


  /**
    * Compute minimal cutsets with Pseudo-Boolean method
    *
    * @param max maximal cutset size if defined, no bound otherwise
    * @return the set of minimal cutsets
    */
  def enumerateCutUntil(max: Option[Int]): Iterable[Set[InstBoolIdent]] = {
    max match {
      case Some(m: Int) => cutPhiPB.computeCutsUntil(m)
      case None => cutPhiPB.computeAllCuts()
    }
  }

  val profilerCutEnumeration = new Profiler[Option[Int], Iterable[Set[InstBoolIdent]]] {
    def runWithProfile(i: Option[Int])(iterations: Int = 1): (Iterable[Set[InstBoolIdent]], ExecutionTimes) = {
      cutPhiPB.phiCutEnumerationProfiler.runWithProfile(i)(iterations)
    }
  }
}
