package analyzers

import ast.dsl.{CompAlternative, CompInstanceIdent}
import interpreter.TraceInterpreter
import util.analyzers._
import util.analyzers.analyzerTypes.Real

/**
  * Trait containing DSE problem resolution with HIPHOPS and SMT
  */
trait DSEProblemSolverAll extends DSEProblemSolverKCRSim with DSEProblemSolverHipHOPS {
  self: Parsing with SMTTranslation with SubstitutionAnalyzerSim with STMDDBuilderSim with TraceInterpreter =>

  override type Solution = Map[CompInstanceIdent, CompAlternative]

  /**
    * Solve DSE method with custom requirements and a given method
    *
    * @param method resolution method
    * @param rReq   optionnal reliability requirement
    * @param cReq   optionnal order requirement
    * @return solution if any
    */
  def solveWith(method: DesignSpaceExplorationMethod, rReq: Option[Real] = None, cReq: Option[Int] = None): Option[Solution] = {
    method match {
      case GeneticExploration => solveWithHipHOP(for (r <- rReq) yield r.toDouble.toString)
      case SMTExploration => solveConfWithReq(rReq, cReq)
    }
  }

  val designSpaceExplorationProfiler = new Profiler[DesignSpaceExplorationMethod, Option[Solution]] {
    override def runWithProfile(i: DesignSpaceExplorationMethod)(iterations: Int = 1): (Option[Solution], ExecutionTimes) = {
      i match {
        case GeneticExploration => designSpaceExplorationHipHopProfiler.runWithProfile()(iterations)
        case SMTExploration => designSpaceExplorationProfilerKCR.runWithProfile()(iterations)
      }
    }
  }
}
