package analyzers

import util.analyzers.SubstitutionVar


/**
  * Trait for DSE solving using SMT based STMDD computation
  */
trait DSEProblemSolverKCRSMT extends DSEProblemSolverKCR with ConfigurationAnalyzer with SafetyTheory {
  self: Parsing with SMTTranslation with SubstitutionAnalyzerSMT with STMDDBuilderSMT =>

  /**
    * Generate substitution variables for component instance in the system
    *
    * @return list of substitution var
    */
  def generateDecisionVars(): List[SubstitutionVar] = {
    rootCompInstsInOrder.collect({
      case inst if rootDef.namespace.locateDef(inst).nonEmpty =>
        substitutions.find(sub => sub.sub.instance.path.mkString(".") == inst.path.mkString(".")) match {
          case Some(sub) =>
            val allAlts = sub.sub.init +: sub.sub.alternatives
            val sort = _SMTContext.mkEnumSort("EnumSort" + allAlts.size, allAlts.map(alt => alt.compId.baseId.name + alt.lambdaValues.map(p => p._2).mkString(",")): _*)
            val slctVar = _SMTContext.mkConst(sub.sub.instance.path.mkString("."), sort)
            SubstitutionVar(inst, slctVar, sort)
          case None => //if no subsitution possible then create a set with only one possible value None
            val sort = _SMTContext.mkEnumSort("EnumSort1", "initial")
            val slctVar = _SMTContext.mkConst(inst.path.mkString("."), sort)
            SubstitutionVar(inst, slctVar, sort)
        }
    })
  }
}
