package analyzers

import interpreter.TraceInterpreter
import util.analyzers.SubstitutionVar

/**
  * Trait for DSE solving using Interpreter based STMDD computation
  */
trait DSEProblemSolverKCRSim extends DSEProblemSolverKCR with ConfigurationAnalyzer with SafetyTheory {
  self: Parsing
    with SMTTranslation
    with SubstitutionAnalyzerSim
    with STMDDBuilderSim
    with TraceInterpreter =>

  /**
    * Generate substitution variables for component instance in the system
    *
    * @return list of substitution var
    */
  def generateDecisionVars(): List[SubstitutionVar] = {
    rootCompInstsInOrder.collect({
      case inst if substitutions.contains(OrderedCompInstIdent(inst)) =>
        val sub = conf.designSpace.find(s => s.instance == inst).get
        val allAlts = sub.init +: sub.alternatives
        val sort = _SMTContext.mkEnumSort("EnumSort" + allAlts.size, allAlts.map(alt => alt.compId.baseId.name + alt.lambdaValues.map(p => p._2).mkString(",")): _*)
        val slctVar = _SMTContext.mkConst(inst.toString, sort)
        SubstitutionVar(inst, slctVar, sort)
    })
  }
}
