package analyzers

import ast.dsl.ConfigurationDeclaration
import util.analyzers.{Console, ParsingFailure}
import util.io.LoadDSLFiles

import scala.util.{Failure, Success}

trait FileParsing extends Parsing {
  self: Console =>

  val fileNames: List[String]
  val confId:String
  lazy val conf:ConfigurationDeclaration = getConfFromFiles(fileNames,confId)

  /**
    * Build KCR AST from files
    * @param files KCR files
    * @param confId configuration to load
    * @return configuration declaration if parsing successful, none otherwise
    */
  private def getConfFromFiles(files: List[String], confId: String): ConfigurationDeclaration = {
    parsers.dsl.CLIParser.parseConfigIdent(confId) match {
      case Success(correctConfId) =>
        LoadDSLFiles.applyAndInlineOrGetErrors(files, correctConfId) match {
          case (Some(result), _) =>
            result
          case (None, Some(errors)) =>
            printError(errors)
            throw ParsingFailure(errors)
        }
      case Failure(e) =>
        printError(e.getMessage)
        throw ParsingFailure(e.getMessage)
    }
  }
}
