package analyzers

import ast.dsl.InstBoolIdent
import util.analyzers.{ExecutionTimes, Phi, PhiCutEnumerationIterativeSAT, Profiler}

trait IterativeCutEnumerator extends CutEnumerator {
  self: Parsing with SMTTranslation =>
  private lazy val cutPhiIterative = new Phi(phi) with PhiCutEnumerationIterativeSAT

  /**
    * Compute cutsets with SAT Iterative method
    *
    * @param max maximal cutset size if defined, no bound otherwise
    * @return the set of minimal cutsets
    */
  def enumerateCutUntil(max: Option[Int]): Iterable[Set[InstBoolIdent]] = {
    max match {
      case Some(m: Int) => cutPhiIterative.computeCutsUntil(m)
      case None => cutPhiIterative.computeAllCuts()
    }
  }

  val profilerCutEnumeration = new Profiler[Option[Int], Iterable[Set[InstBoolIdent]]] {
    def runWithProfile(i: Option[Int])(iterations: Int = 1): (Iterable[Set[InstBoolIdent]], ExecutionTimes) = {
      cutPhiIterative.phiCutEnumerationProfiler.runWithProfile(i)(iterations)
    }
  }
}

