package analyzers

import java.io.File

import ast.dsl.InstBoolIdent
import translators.DSLOpenPSATranslator
import util.analyzers.Z3ExprTranslators
import util.analyzers.analyzerTypes.Real

import scala.xml.dtd.DocType

/**
  * Trait for OpenPSA exportation
  */
trait OpenPSAExporter extends Exporter with Z3ExprTranslators {
  self: Parsing with SMTTranslation =>

  /**
    * Translate structure function to OpenPSA fault tree
    *
    * @param file destination file
    */
  def exportToOpenPSA(file: File): Unit = {
    val lambdas = conf.designSpace.flatMap(s => {
      s.init.lambdaValues.map(p =>
        InstBoolIdent(p._1, s.instance) -> (Real(p._2.repr), for (t <- conf.duration) yield t.repr.toDouble))
    }).toMap
    val varMap = phi.events.map(p => {
      val param = lambdas(p._1)
      p._2 -> (p._1.toString, param._1, param._2)
    })
    val tr = DSLOpenPSATranslator.boolExprToPSA(toNNF(phi.expr, phi.events.values.toList, _SMTContext), varMap)
    scala.xml.XML.save(file.getAbsolutePath, tr, xmlDecl = true, doctype = DocType("open-psa"))
  }
}
