package analyzers

import ast.dsl.{CompIdent, CompInstanceIdent, ComponentDeclaration, ConfigurationDeclaration}
import util.analyzers.{Console, ParsingFailure}
import util.io.LoadDSLFiles

import scala.util.{Failure, Success}

trait Parsing {

  val conf:ConfigurationDeclaration

  /**
    * Locate component definition from a given component instance identifier
    *
    * @param inst the instance identifier
    * @return the component declaration
    */
  def locateCompDef(inst: CompInstanceIdent): ComponentDeclaration = {
    val root = conf.namespace.locate(conf.rootCompId)
    val compId = root.namespace.locateDef(inst).head.rhs.compId
    root.namespace.locate(compId)
  }

  /**
    * Locate component declaration from component identifier
    *
    * @param compId the component identifier
    * @return the component declaration
    */
  def locateCompDef(compId: CompIdent): ComponentDeclaration = {
    val root = conf.namespace.locate(conf.rootCompId)
    root.namespace.locate(compId)
  }

  /**
    * the component declaration of the system
    */
  lazy val rootDef: ComponentDeclaration = locateCompDef(conf.rootCompId)

  /**
    * the component instances in the system sorted by topological order
    */
  lazy val rootCompInstsInOrder: List[CompInstanceIdent] =
    rootDef.namespace.compInstanceDepGraph.inOrder.map(_.data).toList


}



