package analyzers

import util.analyzers.{ExecutionTimes, Profiler}
import util.analyzers.analyzerTypes.Real

/**
  * Reliability computation trait
  */
trait ReliabilityComputer extends SafetyIndicatorComputer with AcceptableReliabilityAnalyzer {
  self: Parsing with SMTTranslation =>

  lazy val reliability: Real = computeIniRBDD()

  val profilerReliabilityComputation = new Profiler[Unit, Real] {
    override def runWithProfile(i: Unit)(iterations: Int = 1): (Real, ExecutionTimes) = {
      val start = System.nanoTime()
      val r = computeIniRBDD()
      val t = System.nanoTime() - start
      (r, ExecutionTimes(0, t, 0))
    }
  }
}