package analyzers

import ast.dsl.InstBoolIdent
import util.analyzers.{ExecutionTimes, Phi, PhiCutEnumerationIterativeSMT, Profiler}

/**
  * Cutsets computation with Iteration SMT method
  */
trait SMTCutEnumerator extends CutEnumerator {
  self: Parsing with SMTTranslation =>

  private lazy val cutPhiSMT = new Phi(phi) with PhiCutEnumerationIterativeSMT

  /**
    * Compute minimal cutsets with Iteration SMT method
    *
    * @param max maximal cutset size if defined, no bound otherwise
    * @return the set of minimal cutsets
    */
  def enumerateCutUntil(max: Option[Int]): Iterable[Set[InstBoolIdent]] = {
    max match {
      case Some(m: Int) => cutPhiSMT.computeCutsUntil(m)
      case None => cutPhiSMT.computeAllCuts()
    }
  }

  val profilerCutEnumeration = new Profiler[Option[Int], Iterable[Set[InstBoolIdent]]] {
    def runWithProfile(i: Option[Int])(iterations: Int = 1): (Iterable[Set[InstBoolIdent]], ExecutionTimes) = {
      cutPhiSMT.phiCutEnumerationProfiler.runWithProfile(i)(iterations)
    }
  }
}

