package analyzers

import util.analyzers.analyzerTypes._
import ast.dsl._
import com.microsoft.z3.{AST => Z3AST, BoolExpr => Z3BoolExpr, Context => Z3Context, Expr => Z3Expr, Status => Z3Status, Symbol => Z3Symbol, _}
import interpreter.TraceInterpreter
import interpreter.TraceInterpreter._
import net.sf.javabdd.BDD
import util.analyzers.InfNumericImplicits._
import util.analyzers.InfNumericTypes._
import util.analyzers.MeasureTypes.CardMeasure
import util.analyzers._
import util.translators.FlowKey

import scala.collection.immutable.BitSet
import scala.reflect.ClassTag
import scala.util.control.TailCalls._
import scala.util.{Failure, Success}
import scalaz.Memo

/**
  * Created by kdelmas on 13/04/16.
  */

/**
  * Base trait for computation of the STMDD
  */
trait STMDDBuilder extends ConfigurationAnalyzer {
  self: Parsing =>

  /**
    * Case class encoding the label on the STMDD edges
    *
    * @param trace the component trace
    * @param unr   the probability array
    * @param card  the order array
    */
  case class STMDDEdgeLabel private(trace: OrderedCompTrace, unr: Array[Real], card: Array[InfInt]) extends Ordered[STMDDEdgeLabel] {
    lazy val maxUnr: Real = unr.max
    lazy val maxCard: InfInt = card.max
    lazy val minUnr: Real = unr.min
    lazy val minCard: InfInt = card.min

    lazy val lessCardThan: (CardMeasure) => BitSet = Memo.mutableHashMapMemo {
      obj => {
        BitSet(card.zipWithIndex.collect({ case (c, index) if CardMeasure(c) < obj => index }): _*)
      }
    }

    override def toString: String = trace.toString

    override lazy val hashCode: Int = trace.hashCode

    override def equals(o: scala.Any): Boolean = {
      o match {
        case that: STMDDEdgeLabel =>
          trace == that.trace && unr.sameElements(that.unr) && card.sameElements(that.card)
        case _ => false
      }
    }

    def compare(that: STMDDEdgeLabel): Int = {
      trace.compare(that.trace)
    }
  }

  /**
    * the MDD factory used to build the STMDD
    */
  val mddFactory: GenMDDFactory[OrderedCompInstIdent,
                                Set,
                                STMDDEdgeLabel,
                                OrderedSymbol,
                                Set[STMDDEdgeLabel]] with MDDPrinter[OrderedCompInstIdent,
                                                                      Set,
                                                                      STMDDEdgeLabel,
                                                                      OrderedSymbol,
                                                                      Set[STMDDEdgeLabel]]

  /**
    * the STMDD of the system
    */
  val STMDDD: mddFactory.MDD

}


