/** $Id$
  *
  * @author $Author$
  * @date $Date$
  **/

package ast.dsl

import namespace.dsl.Namespace

sealed trait ASTException extends Exception

/** Thrown when accessing a TargetField that has not been set yet. */
case class TargetNotSet(node: ASTNode) extends ASTException

/** Adds a pointed to a target declaration to a node. */
trait TargetDeclField[D <: ProgramDeclaration] {
  this: ASTNode =>
  private var _target: Option[D] = None

  def target_(t: D): Unit = {
    _target = Some(t)
  }

  def target: D = _target getOrElse {
    throw TargetNotSet(this)
  }

  def targetOpt: Option[D] = _target
}

/** Thrown when accessing a NamespaceField that has not been set yet. */
case class NamespaceNotSet(node: ASTNode) extends ASTException

/** Adds a namespace field to a node. */
trait NamespaceField[N <: Namespace] {
  this: ASTNode =>
  private var _namespace: Option[N] = None

  def namespace_(n: N): Unit = {
    _namespace = Some(n)
  }

  def namespace: N = _namespace getOrElse {
    throw NamespaceNotSet(this)
  }

  def namespaceOpt: Option[N] = _namespace
}

/** Thrown when accessing a ParentField that has not been set yet. */
case class ParentNotSet(node: ASTNode) extends ASTException

/** Adds a parent node to a node. */
trait ParentField[A <: ASTNode] {
  this: ASTNode =>
  private var _parentOpt: Option[A] = None

  def parent_(p: A): Unit = {
    _parentOpt = Some(p)
  }

  def parent: A = _parentOpt.getOrElse {
    throw ParentNotSet(this)
  }

  def parentOpt: Option[A] = _parentOpt
}

/** Magic object used to circumvent type erasure. */
trait ParentEitherFieldAMagic

/** Magic object used to circumvent type erasure. */
trait ParentEitherFieldBMagic

/** Adds a disjunction of parents to a node. */
trait ParentEitherField[A <: ASTNode, B <: ASTNode] {
  this: ASTNode =>
  private var _parentOpt: Option[Either[A, B]] = None

  def parent_(p: A)(implicit evt: ParentEitherFieldAMagic): Unit = {
    _parentOpt = Some(Left(p))
  }

  def parent_(p: B)(implicit evt: ParentEitherFieldBMagic): Unit = {
    _parentOpt = Some(Right(p))
  }

  def parent: Either[A, B] = _parentOpt.getOrElse {
    throw ParentNotSet(this)
  }

  def parentOpt: Option[Either[A, B]] = _parentOpt
}
