package ast.dsl

import java.io.{BufferedWriter, File, FileWriter}

/**
  * Created by kdelmas on 12/12/16.
  */


/**
  * Generate a new system by applying substitution
  */
object Transformer {

  /**
    * Generate new system by replacing the type of the component instance in the system by the one given
    * in the map by
    * @param in the initial package containing the definitions
    * @param of the configuration containing the root system to modify
    * @param by the transformation map
    * @return the new package containing modified system
    */
  def replaceInstanceComponent(in: PackageDeclaration,
                               of: ConfigurationDeclaration,
                               by: Map[CompInstanceIdent, CompAlternative]): PackageDeclaration = {

    val rootComp = of.namespace.locate(of.rootCompId)
    val newDefinitions: List[FlowDefinition] = rootComp.body.definitions.map({
      case fdef@CompCallFlowDefinition(mlhs, mrhs) => by.find(p => p._1.path.last == mrhs.instId) match {
        case Some(pair) =>
          fdef.copy(rhs = mrhs.copy(compId = pair._2.compId))
        case None => fdef
      }
      case other => other
    })
    val newComponent = rootComp.copy(body = rootComp.body.copy(definitions = newDefinitions))
    val newDesignSpace = of.designSpace.map(compSub => {
      by.get(compSub.instance) match {
        case Some(alt) => compSub.copy(init = alt, alternatives = Nil)
        case None => compSub
      }
    })
    in.copy(declarations = in.declarations.map(decl => {
      if (decl == rootComp)
        newComponent
      else if (decl == of)
        of.copy(designSpace = newDesignSpace)
      else decl
    }))
  }

  def writeSolutionFile(file: File,
                        program: PackageDeclaration,
                        conf: ConfigurationDeclaration,
                        replaceMap: Map[CompInstanceIdent, CompAlternative]): Unit = {

    val sltProg = replaceInstanceComponent(program, conf, replaceMap)
    if (!file.exists()) {
      file.createNewFile()
    }
    val fw: FileWriter = new FileWriter(file.getAbsoluteFile)
    val bw: BufferedWriter = new BufferedWriter(fw)
    val printer = new printing.dsl.Printer(bw)
    printer.asRoot(sltProg)
    bw.close()
    fw.close()
  }
}
