/** $Id: InliningCache.scala 1433 2017-10-19 14:06:15Z kdelmas $
  *
  * @author $Author: kdelmas $
  * @date $Date: 2017-10-19 16:06:15 +0200 (jeu. 19 oct. 2017) $ 
  **/

package inlining.dsl

import ast.dsl._

import scala.collection.mutable

/** Mutable inlining cache that stores new AST elements generated during the inlining process. */
class InliningCache() {

  /** Type declarations generated during the inling, retains the insertion order. */
  val typeDecls: mutable.LinkedHashMap[TypeIdent, TypeDeclaration] = mutable.LinkedHashMap.empty

  /** Primary components pulled in during component inlining, retains the insertion order. */
  val primaryCompDecls: mutable.LinkedHashMap[CompIdent, ComponentDeclaration] = mutable.LinkedHashMap.empty

  /** Local flow declarations generated during the inling, retains the insertion order. */
  val flowDecls: mutable.LinkedHashMap[BasicFlowIdent, FlowDeclaration] = mutable.LinkedHashMap.empty

  /** Output flow declarations generated during the inlining, retains the insertion order. */
  val outputDecls: mutable.LinkedHashMap[BasicFlowIdent, FlowDeclaration] = mutable.LinkedHashMap.empty

  /** Input flow declarations generated during the inlining, retains the insertion order. */
  val inputDecls: mutable.LinkedHashMap[BasicFlowIdent, FlowDeclaration] = mutable.LinkedHashMap.empty

  /** Event declarations generated during the inlining, retains the insertion order. */
  val evtDecls: mutable.LinkedHashSet[BoolIdent] = mutable.LinkedHashSet.empty

  /** Flow definitions generated during the inlining, retains the insertion order. */
  val definitions: mutable.LinkedHashSet[FlowDefinition] = mutable.LinkedHashSet.empty

  /** SingleFlowDefinitions generated during the inlining, keyed under BasicFlowIdent:s, retains the insertion order. */
  val singleFlowDefinitions: mutable.LinkedHashMap[BasicFlowIdent, SingleFlowDefinition] = mutable.LinkedHashMap.empty

  /** CompCallFloDefinitions generated during the inlining, keyed under CompInstanceId:s, retains the insertion order. */
  val callDefinitions: mutable.LinkedHashMap[CompInstanceIdent, CompCallFlowDefinition] = mutable.LinkedHashMap.empty

  def print(): Unit = {
    println("Types")
    typeDecls.values.foreach {
      println
    }

    println("Primary Components")
    primaryCompDecls.values.foreach {
      println
    }

    println("Local Flows")
    for ((key, value) <- flowDecls) {
      println(value.toString)
    }

    println("Output Flows")
    outputDecls.values.foreach {
      println
    }

    println("Definitions")
    definitions.foreach {
      println
    }

  }

}
