/** $Id: Action.scala 1433 2017-10-19 14:06:15Z kdelmas $
  *
  * @author $Author: kdelmas $
  **/

package namespace.dsl

import scalaz.\/
import scalaz.syntax.either._


/** Base trait for namespace actions. */
trait Action extends (() => (List[Action], List[NamespaceException]))

/** Runs a build action to completion using a trampoline.  All actions
  * resulting from the action are run to completion and the list of
  * all non-fatal exceptions triggered during processing is returned,
  * unless a single fatal exceptions is raised in which case it is
  * returned.
  */
object RunAction extends (Action => \/[NamespaceExceptions, List[NamespaceWarning]]) {

  def apply(action: Action): \/[NamespaceExceptions, List[NamespaceWarning]] = {
    var _exceptions: List[NamespaceException] = Nil
    var _actions: List[Action] = action :: Nil
    while (_actions.nonEmpty) {
      val (nextActions, exceptions) = _actions.head()
      _actions = nextActions ::: _actions.tail
      _exceptions :::= exceptions
    }
    val fatal = _exceptions collect { case e: NamespaceFatal => e }
    if (fatal.isEmpty) {
      {
        _exceptions collect { case e: NamespaceWarning => e }
      }.right
    } else {
      NamespaceExceptions(_exceptions).left
    }
  }

}
