/** $Id: BuildConfigCallGraphs.scala 1433 2017-10-19 14:06:15Z kdelmas $
  *
  * @author $Author: kdelmas $
  **/

package namespace.dsl

import ast.dsl._

import scalaz.\/

/** Trampoline that builds and checks the namespace of a program. 
  *   - builds call graphs for all root nodes of root components of all declarations found under the given package
  * */
object BuildConfigCallGraphs extends (PackageDeclaration => \/[NamespaceExceptions, List[NamespaceWarning]]) {

  def apply(pkg: PackageDeclaration): \/[NamespaceExceptions, List[NamespaceWarning]] = {
    for {
      compCycles <- RunAction(BuildCallGraphPackage(pkg))
    } yield {
      compCycles
    }
  }

}

/** Builds call graphs for root nodes of all configurations found under the given package. */
case class BuildCallGraphPackage(pkg: PackageDeclaration) extends Action {

  def apply(): (List[Action], List[NamespaceException]) = {
    val namespace = pkg.namespace
    val actions =
      namespace.packages.values.map {
        BuildCallGraphPackage
      }.toList :::
        namespace.configurations.values.map { conf => BuildCallGraphComp(conf.namespace.locate(conf.rootCompId)) }.toList
    (actions, Nil)
  }

}


/** Builds the call graph of a given component considered as root. */
case class BuildCallGraphComp(comp: ComponentDeclaration) extends Action {

  def apply(): (List[Action], List[NamespaceException]) = {
    val depGraph = new CompInstanceDepGraph(comp)
    val exceptions = depGraph.check
    comp.namespace.compInstanceDepGraph_(depGraph)
    val actions = Nil
    (actions, exceptions)
  }

}

