/** $Id: CheckCompCycles.scala 1433 2017-10-19 14:06:15Z kdelmas $
  *
  * @author $Author: kdelmas $
  **/

package namespace.dsl

import ast.dsl._

import scalaz.\/

/** Trampoline that builds and checks the namespace of a program. 
  *   - checks for cyclic component dependencies
  * */
object CheckCompCycles extends (PackageDeclaration => \/[NamespaceExceptions, List[NamespaceWarning]]) {

  def apply(pkg: PackageDeclaration): \/[NamespaceExceptions, List[NamespaceWarning]] = {
    for {
      compCycles <- RunAction(CheckCyclesRootPackage(pkg))
    } yield {
      compCycles
    }
  }

}

/** Check cyclic component definitions from the root package. */
case class CheckCyclesRootPackage(rootPkg: PackageDeclaration) extends Action {

  def apply(): (List[Action], List[NamespaceException]) = {
    val depGraph = new ComponentDepGraph(rootPkg)
    val exceptions = depGraph.check
    rootPkg.namespace.componentDepGraph_(depGraph)
    val actions = Nil
    (actions, exceptions)
  }

}
