/** $Id: MutableFields.scala 1433 2017-10-19 14:06:15Z kdelmas $
  *
  * @author $Author: kdelmas $
  **/

package namespace.dsl

/* Mutable fields added to the namespace objects by different passes. */


/** Dependency graph for all flows found in a component. */
trait FlowDepGraphField {
  this: ComponentNamespace =>
  private var _flowDepGraph: Option[FlowDepGraph] = None

  def flowDepGraph_(g: FlowDepGraph): Unit = {
    _flowDepGraph = Some(g)
  }

  def flowDepGraph: FlowDepGraph = _flowDepGraph getOrElse {
    throw DepGraphNotSet(this, "flowDepGraph")
  }

  def flowDepGraphOpt: Option[FlowDepGraph] = _flowDepGraph
}

/** Dependency graph for all components found under the root package. */
trait ComponentDepGraphField {
  this: PackageNamespace =>
  private var _componentDepGraph: Option[ComponentDepGraph] = None

  def componentDepGraph_(g: ComponentDepGraph): Unit = {
    _componentDepGraph = Some(g)
  }

  def componentDepGraph: ComponentDepGraph = _componentDepGraph getOrElse {
    throw DepGraphNotSet(this, "componentDepGraph")
  }

  def componentDepGraphOpt: Option[ComponentDepGraph] = _componentDepGraph
}

/** Depedency graph for all components found under the root node of a configuration (ie, the call graph). */
trait CompInstanceDepGraphField {
  this: Namespace =>
  private var _compInstanceDepGraph: Option[CompInstanceDepGraph] = None

  def compInstanceDepGraph_(g: CompInstanceDepGraph): Unit = {
    _compInstanceDepGraph = Some(g)
  }

  def compInstanceDepGraph: CompInstanceDepGraph = _compInstanceDepGraph getOrElse {
    throw DepGraphNotSet(this, "compInstanceDepGraph")
  }

  def compInstanceDepGraphOpt: Option[CompInstanceDepGraph] = _compInstanceDepGraph
}

/** Set of all type declarations used in a given namespace. */
trait UsedTypesSetField {
  this: Namespace =>
  private var _usedTypesSet: Option[TypeSetType] = None

  def usedTypesSet_(g: TypeSetType): Unit = {
    _usedTypesSet = Some(g)
  }

  def usedTypesSet: TypeSetType = _usedTypesSet getOrElse {
    throw new Exception(s"usedTypesSet field not set yet for ${this.source.toString}")
  }

  def usedTypesSetOpt: Option[TypeSetType] = _usedTypesSet
}

