package old

/**
  * Created by kdelmas on 03/05/17.
  */

object AntColony extends App {

  case class Ant(path: List[(Int, Int)]) {
    val best = if (path.nonEmpty)
      candidate.filter(kv => path.exists(arc => kv._1 == arc._1 || kv._1 == arc._2)).maxBy(kv => kv._2)
    else
      1 -> candidate(1)
  }

  val nbAnts = 100

  // desirability = exp(10*(Rend-Rstart))
  val desirability = Map(
    (1, 2) -> 0.09,
    (2, 1) -> -0.09,
    (2, 3) -> 0.009,
    (3, 2) -> -0.009,
    (3, 4) -> -0.05,
    (4, 3) -> 0.05,
    (4, 5) -> 0.09,
    (5, 4) -> -0.09,
    (5, 6) -> -0.05,
    (6, 5) -> 0.05,
    (6, 7) -> -0.09,
    (7, 6) -> 0.09,
    (7, 1) -> -0.009,
    (1, 7) -> 0.009).mapValues(v => Math.exp(v * 10))

  //candidate R
  val candidate = Map(
    1 -> 0.79,
    2 -> 0.88,
    3 -> 0.889,
    4 -> 0.849,
    5 -> 0.939,
    6 -> 0.889,
    7 -> 0.799
  )

  val Rgoal = 0.9

  val evapFactor = 0.5

  val nbMouv = 3

  //pheromone values
  var pheromone = desirability.mapValues(v => 1.0)

  def updatePheromone(ants: List[Ant]): Unit = {
    //evaporation
    pheromone = pheromone.mapValues(v => v * evapFactor)
    //deposit of pheromone
    for (ant <- ants) {
      val delta = Math.exp((ant.best._2 - Rgoal) / (Rgoal - candidate(1)) * 5)
      for (p <- ant.path) {
        pheromone += (p -> (pheromone(p) + delta))
      }
    }
  }

  def probaOf(arc: (Int, Int)): Double = {
    pheromone(arc) * desirability(arc) / pheromone.collect({ case kv if kv._1._1 == arc._1 => pheromone(kv._1) * desirability(kv._1) }).sum
  }

  var antMap = candidate.map(kv => if (kv._1 == 1) kv._1 -> (1 to nbAnts).toList.map(v => Ant(Nil)) else kv._1 -> List[Ant]())

  def iteration(): Unit = {

    for (i <- 1 to nbMouv) {
      var antMap2 = antMap.mapValues(v => List[Ant]())
      for (c <- antMap) {
        pheromone.keys.filter(k => k._1 == c._1).foreach(arc => {
          val newAnts: List[Ant] = antMap2(arc._2) ::: antMap(arc._1).slice(0, (probaOf(arc) * antMap(arc._1).size).toInt).map(a => a.copy(path = a.path :+ arc))
          antMap2 += (arc._2 -> newAnts)
        })
      }
      antMap = antMap2

    }

  }

  for (i <- 1 to 4) {
    antMap = candidate.map(kv => if (kv._1 == 1) kv._1 -> (1 to nbAnts).toList.map(v => Ant(Nil)) else kv._1 -> List[Ant]())
    iteration()
    updatePheromone(antMap.values.flatten.toList)
    println("ant per best candidate :\n" + antMap.valuesIterator.flatten.toList.groupBy(a => a.best._1).map(kv => s"${kv._1} -> ${kv._2.size}").mkString("\n"))
  }


}
