package old

import analyzers._
import ast.dsl.{CompAlternative, CompInstanceIdent}
import com.microsoft.z3._
import translators.translatorsTypes._
import util.analyzers.InfNumericTypes._
import util.analyzers.MeasureTypes.CardMeasure
import util.analyzers.{CardMeasure, Measure, POSITIVE_INFINITY, SubstitutionVar}
import util.stack.immutable.{AssertionStack => KeepStack}

import scala.collection.immutable.{BitSet, HashMap}
import scala.collection.mutable
import scalaz.Memo

/**
  * Created by kdelmas on 14/04/16.
  */
/**
  * cardinality theory trait based on MDD analysis
  */
@deprecated("old version of safety theory not using clause minimisation","KCR analyser 1.0")
trait CardinalityTheory {
  self: Parsing with SMTTranslation with STMDDBuilder =>

  /**
    * compute the cardinality of a substitution choice
    *
    * @param choice the substitution choice
    * @return the over, under and exact cardinality (if computable)
    */
  def computeCard(choice: Map[CompInstanceIdent, CompAlternative]): CardMeasure = {
    computeCardRec(STMDDD, choice)
  }

  /**
    * find the alternative choice concerning the component labelled by the mdd node
    * Warning: the alternative choice must be in choice, otherwise the method returns -1 index
    *
    * @param choice the set of alternative choices
    * @param of     the mdd node
    * @return the index in the substitution choices
    */
  private def getChoiceCardIndex(choice: Seq[(PathIdentType, Option[Int])], of: mddFactory.MDDNode): Int = {
    choice.indexWhere(p => p._1 == of.param.value.path)
  }

  @deprecated("very fragile, change it !","KCR analyser 1.0")
  private def getAlternativeIndex(inst: CompInstanceIdent, alt: CompAlternative): Int = {
    val sub = conf.designSpace.find(s => s.instance.path == inst.path).get
    (sub.init +: sub.alternatives).indexOf(alt)
  }

  /**
    * Recursively compute the cardinality of a mdd node according to a substitution choice which is
    * card(node)=min_{s \in node.kids} (card(s.tr,choice(node))+card(s.son))
    *
    * @param node   mdd node
    * @param choice substitution chocie
    * @return the over, under and exact cardinality (if computable)
    */
  def computeCardRec(node: mddFactory.MDD, choice: Map[CompInstanceIdent, CompAlternative]): CardMeasure = {
    import util.analyzers.InfNumericImplicits._
    node match {
      case n: mddFactory.MDDNode =>
        //retrieving substitution choice for the current node
        val nodeChoice = for (compAlt <- choice.find(p => n.param.value.path == p._1.path)) yield {
          getAlternativeIndex(n.param.value, compAlt._2)
        }
        nodeChoice match {
          case Some(bit) =>
            //computing sons' exact, over and under approximations of cardinality
            val sonCard = n.map(pair => pair._1 -> computeCardRec(pair._2, choice))
            val over = sonCard.map(kv => kv._2.up + kv._1.card(bit)).min
            val under = sonCard.map(kv => kv._2.low + kv._1.card(bit)).min
            val real: Option[InfInt] = sonCard.foldLeft[Option[InfInt]](Some(POSITIVE_INFINITY))((acc, son) => {
              for (m <- acc; e <- son._2.exact) yield InfIntIsNumeric.min(m, e)
            })
            Measure(over, under, real)
          case None =>
            val sonCard = n.kids.mapValues(kid => computeCardRec(kid, choice))
            val over = sonCard.map(kv => kv._2.up + kv._1.maxCard).min
            val under = sonCard.map(kv => kv._2.low + kv._1.minCard).min
            Measure(over, under, None)
        }
      case l: mddFactory.Terminal =>
        //if the node is a terminal then we reach the failure condition thus card(t)=0
        Measure(0)
    }
  }

  /**
    * Recursively check if the cardinality budget can be dispatched for a given mdd node and substitution choice
    *
    * @param node   mdd node
    * @param choice substitution chocie
    * @param remain cardinality budget which can be spent be the node and its kids
    * @return the status of the check, the effective cardinality computed and the substitutions informations used to
    *         decide if the cardinality budget is fulfilled
    */
  private val isSafeRec: ((mddFactory.MDD, Seq[(PathIdentType, Option[Int])], CardMeasure)) => (Status, CardMeasure, HashMap[PathIdentType, BitSet]) = Memo.mutableHashMapMemo {
    import util.analyzers.InfNumericImplicits._
    (t: (mddFactory.MDD, Seq[(PathIdentType, Option[Int])], CardMeasure)) => {
      val (node, choice, remain) = t
      node match {
        case n: mddFactory.MDDNode =>
          //choice for the node
          val nodeChoiceIndex = getChoiceCardIndex(choice, n)
          //choice for sons
          val choiceTail = choice.drop(nodeChoiceIndex + 1)
          //compute the new budget for each son
          val newRemains: HashMap[STMDDEdgeLabel, CardMeasure] =
            choice(nodeChoiceIndex)._2 match {
              case Some(bit) =>
                //if the substitution is known then the new budget is remain minus the number of events needed to trigger the
                //labelled trace
                HashMap(n.map(pair => pair._1 -> (remain - pair._1.card(bit))).toSeq: _*)
              case None =>
                //if the substitution choice is not known then the new budget is bounded by taking the minimal/maximal
                //number of events needed to trigger the labelled trace among all possible substitutions
                HashMap(n.map(pair => pair._1 -> Measure(remain.up - pair._1.minCard,
                  remain.low - pair._1.maxCard,
                  None)).toSeq: _*)
            }
          val cardRes = newRemains.foldLeft((CardMeasure(99), HashMap[Status, HashMap[PathIdentType, BitSet]]()))((acc, kv) => {
            //checking the new budget on node's sons
            val sonEval: (Status, CardMeasure, HashMap[PathIdentType, BitSet]) = {
              val (label, sonRemain) = kv
              // ie the budget is negative, then the whole path is acceptable
              if (sonRemain <= 0)
                (Status.SATISFIABLE, {
                  for (bit <- choice(nodeChoiceIndex)._2) yield Measure(label.card(bit))
                }.getOrElse(Measure(label.maxCard, label.minCard, None)), {
                  for (bit <- choice(nodeChoiceIndex)._2) yield HashMap(n.param.value.path -> BitSet(bit))
                }.getOrElse(HashMap()))
              else {
                //if the budget is not dispatched then propagate teh analysis on son
                isSafeRec((n.kids(label), choiceTail, sonRemain)) match {
                  case (Status.UNSATISFIABLE, sonCard, cube) => choice(nodeChoiceIndex)._2 match {
                    case Some(bit) =>
                      // if the son does not ensure the budget then the candidate is not acceptable
                      if (label.maxCard > label.card(bit)) {
                        //if it exists a better substitution choice, then this path could have been acceptable
                        //so all substitution choices which are not better than the actual one are added to the clause
                        (Status.UNSATISFIABLE, sonCard + label.card(bit), cube + (n.param.value.path -> label.lessCardThan(remain - sonCard)))
                      } else {
                        //if the substitution choice is the best we can make then the choice on this node is not
                        // an explanation of the conflict
                        (Status.UNSATISFIABLE, sonCard + label.card(bit), cube)
                      }
                    case None =>
                      //if the substitution choice is not known then the choice on this node is not
                      // an explanation of the conflict
                      (Status.UNSATISFIABLE, sonCard + Measure(label.maxCard, label.minCard, None), cube)
                  }
                  case (Status.UNKNOWN, sonCard, prop) =>
                    //if some substitutions satisfy the budget and others don't then
                    choice(nodeChoiceIndex)._2 match {
                      case Some(bit) =>
                        // if the substitution choice is known then we do not propagate any decisions
                        // indeed a propagation is model => formula, thus adding any information on this node
                        // substitution choice is meaningless (model precondition will be false since we talk about another model)
                        (Status.UNKNOWN, sonCard + label.card(bit), prop)
                      case None =>
                        // if the substitution choice is not known then we reject all possible substitution choices
                        // which would violate the budget allocation at this node level
                        val rejected = label.lessCardThan(remain - sonCard)
                        if (rejected.nonEmpty)
                          (Status.UNKNOWN, sonCard + Measure(label.maxCard, label.minCard, None), prop + (n.param.value.path -> rejected))
                        else
                          (Status.UNKNOWN, sonCard + Measure(label.maxCard, label.minCard, None), prop)


                    }
                  case (Status.SATISFIABLE, sonCard, cube) =>
                    (Status.SATISFIABLE, {
                      for (bit <- choice(nodeChoiceIndex)._2)
                        yield sonCard + label.card(bit)
                    }.getOrElse(sonCard + Measure(label.maxCard, label.minCard, None)),
                      cube)
                }
              }
            }
            //the minimum of sons card is keept, the conclusion are merged
            (acc._1.min(sonEval._2), acc._2.merged(HashMap(sonEval._1 -> sonEval._3))({ case ((k1, v1), (_, v2)) => (k1, v1.merged(v2)({ case a => a._1 })) }))
          })
          if (cardRes._2.contains(Status.UNSATISFIABLE)) {
            // if some sons are unsatisfiables then return the conflict clauses
            (Status.UNSATISFIABLE, cardRes._1, cardRes._2(Status.UNSATISFIABLE))
          } else if (cardRes._2.contains(Status.UNKNOWN)) {
            //if some sons are unknown then return the propagations
            (Status.UNKNOWN, cardRes._1, cardRes._2(Status.UNKNOWN))
          } else {
            //otherwise the sons are ok
            (Status.SATISFIABLE, cardRes._1, cardRes._2(Status.SATISFIABLE))
          }
        case t: mddFactory.Terminal =>
          if (remain <= 0) {
            //if the budget is allocated then the path is acceptable
            (Status.SATISFIABLE, CardMeasure(0), HashMap[PathIdentType, BitSet]())
          } else if (remain > 0) {
            //if the budget is not allocated for a terminal then the failure condition can be triggred without
            // dispatching the budget, thus is not acceptable
            (Status.UNSATISFIABLE, CardMeasure(0), HashMap[PathIdentType, BitSet]())
          } else {
            //if depending on predecessor decision the budget can be fulfilled or not then we dont know
            (Status.UNKNOWN, CardMeasure(0), HashMap[PathIdentType, BitSet]())
          }
      }
    }
  }

  val enumVarSafeCardCache = mutable.HashMap[PathIdentType, SubstitutionVar]()


  /**
    * cardinality theory which returns for a given substitution choice, the status and the propogation and conflicts.
    * The theory try to dispatch on the trace tree a budget corresponding to the cardinality requirement.
    * For cardinality since card(node)=min_{s \in node.kids} (card(s)+card(tr_{node \to s})), then if one path does not
    * fulfill the requirement then the candidate does not fill the requirement.
    * Since the substitution choice is not always complete, an over and under
    * approximations of the budget are computed at each step.
    *
    * @param model subsitution choice
    * @param obj   cardinality requirement
    * @return the status of the dispatch operation and the conflicts/propagations
    */
  def isSafeCard(obj: Int, model: Traversable[(SubstitutionVar, Expr)]): (Status, Seq[Seq[BoolExpr]]) = {
    import util.analyzers.InfNumericImplicits._
    def asCube(cube: HashMap[PathIdentType, BitSet]): Seq[Seq[BoolExpr]] = {
      cube.collect({ case (enumVar, set) if set.nonEmpty =>
        val assign = enumVarSafeCardCache.getOrElseUpdate(enumVar, model.find(p => p._1.inst.path == enumVar).get._1)
        set.map(bit => _SMTContext.mkEq(assign.z3Var, _SMTContext.getValueAtPos(assign, bit).get)).toSeq
      }).toSeq
    }

    def asCNF(cube: HashMap[PathIdentType, BitSet]): Seq[Seq[BoolExpr]] = {
      cube.collect({ case (enumVar, set) if set.nonEmpty =>
        val assign = enumVarSafeCardCache.getOrElseUpdate(enumVar, model.find(p => p._1.inst.path == enumVar).get._1)
        set.map(bit => _SMTContext.mkEq(assign.z3Var, _SMTContext.getValueAtPos(assign, bit).get)).toSeq
      }).toSeq
    }

    isSafeRec((STMDDD, model.map(p => p._1.inst.path -> _SMTContext.getValuePos(p._1, p._2)).toSeq, CardMeasure(obj))) match {
      case (Status.SATISFIABLE, card, cube) => (Status.SATISFIABLE, asCube(cube))
      case (Status.UNSATISFIABLE, card, cube) => (Status.UNSATISFIABLE, asCNF(cube))
      case (Status.UNKNOWN, card, prop) => (Status.UNKNOWN, asCube(prop))
    }
  }
}

trait CardinalityTheorySim extends CardinalityTheory {
  self: Parsing with SMTTranslation with SubstitutionAnalyzerSim with STMDDBuilderSim =>
}

trait CardinalityTheorySMT extends CardinalityTheory {
  self: Parsing with SMTTranslation with SubstitutionAnalyzerSMT with STMDDBuilderSMT =>
}