package old

import java.io.{File, FileWriter}

import analyzers._
import util.analyzers.analyzerTypes._
import ast.dsl.{BoolIdent, CompAlternative, CompInstanceIdent, DecRealLit, StdDSLSymbol}
import com.microsoft.z3.{BoolExpr, Expr, Solver, Status}
import interpreter.TraceInterpreter
import translators.DSLToHipHOPSTranslator
import util.analyzers._
import util.io.{FileManager, SystemInfo}
import util.stack.StackSolverProxy
import util.stack.immutable.{AssertionStack => KeepStack}

import scala.language.postfixOps
import scala.sys.process._
import scala.xml.XML

/**
  * Created by kdelmas on 14/04/16.
  */
class Z3SolverProxy(solver: Solver) extends StackSolverProxy[BoolExpr] {
  def name: String = "Z3SolverProxy"

  def add(a: BoolExpr): Unit = solver.add(a)

  def add(as: List[BoolExpr]): Unit = solver.add(as: _*)

  def push(): Unit = solver.push()

  def pop(n: Int = 1): Unit = solver.pop(n)

  def reset(): Unit = pop(solver.getNumScopes)

  override def toString: String = name
}

@deprecated("old version of design sapce exploration solver","KCR analyser 1.0")
trait DesignSpaceExploration extends ConfigurationAnalyzer {
  self: Parsing =>
  val solution: Option[Map[CompInstanceIdent, CompAlternative]]
}

trait DesignSpaceExplorationAll extends DesignSpaceExplorationKCRSim with DesignSpaceExplorationHipHOPS {
  self: Parsing with SMTTranslation with SubstitutionAnalyzerSim with STMDDBuilderSim with TraceInterpreter =>

  def exploreWith(method: DesignSpaceExplorationMethod, rReq: Option[Real] = None, cReq: Option[Int] = None): Option[Map[CompInstanceIdent, CompAlternative]] = {
    method match {
      case GeneticExploration => exploreWithHipHOP(for (r <- rReq) yield r.toDouble.toString)
      case SMTExploration => explore(rReq, cReq)
    }
  }

  val designSpaceExplorationProfiler = new Profiler[DesignSpaceExplorationMethod, Option[Map[CompInstanceIdent, CompAlternative]]] {
    override def runWithProfile(i: DesignSpaceExplorationMethod)(iterations: Int = 1): (Option[Map[CompInstanceIdent, CompAlternative]], ExecutionTimes) = {
      i match {
        case GeneticExploration => designSpaceExplorationHipHopProfiler.runWithProfile()(iterations)
        case SMTExploration => designSpaceExplorationProfilerKCR.runWithProfile()(iterations)
      }
    }
  }
}

trait DesignSpaceExplorationHipHOPS extends ConfigurationAnalyzer {
  self: Parsing =>

  def exploreWithHipHOP(req: Option[String] = None): Option[Map[CompInstanceIdent, CompAlternative]] = {
    val modelFile = createHipHOPSFiles(req)
    val resFile = computeWithHipHOPS(modelFile)
    backtranslateFile(resFile)
  }

  private def createHipHOPSFiles(req: Option[String] = None): File = {
    val tmpHip = FileManager.addFileToTemp("tempHip.xml")
    DSLToHipHOPSTranslator.translateConfiguration(conf, tmpHip, req)
    tmpHip
  }

  private def locateFirstFile(dir: File, filter: (File) => Boolean): Option[File] = {
    val these = dir.listFiles
    (for (r <- these.find(filter)) yield {
      r
    }) orElse {
      these.filter(_.isDirectory).flatMap(x => locateFirstFile(x, filter)).toList match {
        case Nil => None
        case h :: t => Some(h)
      }
    }
  }

  private def computeWithHipHOPS(hiphopFile: File): File = {
    val outRepertory = FileManager.addFileToTemp("tempHipOutExp")
    val tooPath = HipHOPSInterface.toolPath.toString()
    SystemInfo.userOs match {
      case SystemInfo.Windows =>
        HipHOPSInterface.startProcess(Process(Seq(
          tooPath,
          hiphopFile.getAbsolutePath,
          "idevns=false",
          s"outputdir=${outRepertory.getAbsolutePath}",
          "optimise=true",
          "maxGenerations=10"))).exitValue()
      case _ =>
        HipHOPSInterface.startProcess(Process(Seq(WineInterface.toolPath.toString(),
          tooPath,
          hiphopFile.getAbsolutePath,
          "idevns=false",
          s"outputdir=${outRepertory.getAbsolutePath}",
          "optimise=true",
          "maxGenerations=10"))).exitValue()
    }
    HipHOPSInterface.killInternetExplorer()
    val filter = (f: File) => {
      f.getName.matches("GA.*\\.xml")
    }
    locateFirstFile(outRepertory, filter).get
  }

  private def computeWithHipHOPSWithTime(hiphopFile: File): (File, Long) = {
    val outRepertory = FileManager.addFileToTemp("tempHipOutExp")
    val tooPath = HipHOPSInterface.toolPath.toString()
    SystemInfo.userOs match {
      case SystemInfo.Windows =>
        HipHOPSInterface.startProcess(Process(Seq(
          tooPath,
          hiphopFile.getAbsolutePath,
          "idevns=false",
          s"outputdir=${outRepertory.getAbsolutePath}",
          "optimise=true",
          "maxGenerations=10"))).exitValue()
      case _ =>
        HipHOPSInterface.startProcess(Process(Seq(WineInterface.toolPath.toString(),
          tooPath,
          hiphopFile.getAbsolutePath,
          "idevns=false",
          s"outputdir=${outRepertory.getAbsolutePath}",
          "optimise=true",
          "maxGenerations=10"))).exitValue()
    }
    HipHOPSInterface.killInternetExplorer()
    val filter = (f: File) => {
      f.getName.matches("GA.*\\.xml")
    }
    val result = locateFirstFile(outRepertory, filter).get
    val tXml = XML.loadFile(result.getAbsolutePath) \\ "CoitGA" \ "TimeElapsed"
    val t = (tXml.head.text.toDouble * 1e9).toLong
    (locateFirstFile(outRepertory, filter).get, t)
  }

  import scala.xml._

  private def locateAlternative(i: String, c: String, lambdas: Map[BoolIdent, DecRealLit]): Option[(CompInstanceIdent, CompAlternative)] = {
    for (sub <- conf.designSpace.find(s => s.instance.path.last.name.replace("|", "") == i);
         alt <- sub.alternatives.find(a => a.compId.baseId.name.replace("|", "") == c & a.lambdaValues.toMap == lambdas)
    ) yield {
      (sub.instance, alt)
    }
  }

  private def locateAlternativeFromString(name: String, inst: String): (CompInstanceIdent, CompAlternative) = {
    val splitted = name.split("_")
    val lambdaMap = splitted.tail.map(s => {
      val pair = s.split("!")
      BoolIdent(StdDSLSymbol(Symbol(pair.head))) -> DecRealLit(pair.last.replace("$", "."))
    }).toMap
    locateAlternative(inst, splitted.head, lambdaMap).get
  }

  private def backtranslateFile(resFile: File): Option[Map[CompInstanceIdent, CompAlternative]] = {
    val xml = XML.loadFile(resFile)
    val individuals = xml \\ "Population" \ "Individual"
    val solutionsStrings = individuals.map(n => {
      val comps = n \\ "Component" \ "Name" map (c => c.text)
      val subs = n \\ "Component" \ "Implementation" \ "Name" map (c => c.text)
      val unr = n \\ "Evaluation" \ "Value" text

      (comps.zip(subs), unr)
    })
    val slts = conf.reliability match {
      case Some(req) =>
        solutionsStrings.collect({
          case (slt, unr) if 1 - Real(unr) >= Real(req.repr) => slt.map(p => locateAlternativeFromString(p._2, p._1)).toMap
        })
      case None =>
        solutionsStrings.map(slt => slt._1.map(p => locateAlternativeFromString(p._2, p._1)).toMap)
    }
    slts.headOption
  }

  val designSpaceExplorationHipHopProfiler = new Profiler[Unit, Option[Map[CompInstanceIdent, CompAlternative]]] {
    override def runWithProfile(i: Unit)(iterations: Int = 1): (Option[Map[CompInstanceIdent, CompAlternative]], ExecutionTimes) = {
      val (r, preTime) = time({
        createHipHOPSFiles()
      }, iterations)
      val (r2, pTime) = computeWithHipHOPSWithTime(r)
      val (r3, postTime) = time({
        backtranslateFile(r2)
      }, iterations)
      (r3, ExecutionTimes(preTime, pTime, postTime))
    }
  }
}

trait DesignSpaceExplorationKCR extends DesignSpaceExploration with CardinalityTheory with STMDDBuilder with UnreliabilityTheory {
  self: Parsing with SMTTranslation =>

  lazy val solution = explore()

  val designSpaceExplorationProfilerKCR = new Profiler[Unit, Option[Map[CompInstanceIdent, CompAlternative]]] {
    override def runWithProfile(i: Unit)(iterations: Int = 1): (Option[Map[CompInstanceIdent, CompAlternative]], ExecutionTimes) = {
      val (_, preTime) = time({
        STMDDD
      }, iterations)
      val (result, pTime) = time({
        explore()
      }, iterations)
      (result, ExecutionTimes(preTime, pTime, 0))
    }
  }

  def explore(): Option[Map[CompInstanceIdent, CompAlternative]] = {
    conf.reliability match {
      case Some(unrObj) => conf.minCardValue match {
        case Some(cardObj) => resolvePb(List(
          (model) => isSafeCard(cardObj.repr.toInt, model),
          (model) => isSafeUnrQuickXplain(1 - Real(unrObj.repr), model)
        ))
        case None => resolvePb(List((model) => isSafeUnrQuickXplain(1 - Real(unrObj.repr), model)))
      }
      case None => conf.minCardValue match {
        case Some(cardObj) => resolvePb(List((model) => isSafeCard(cardObj.repr.toInt, model)))
        case None => None
      }
    }
  }

  def exploreWithoutXplain(RObj: Option[Real], cardObj: Option[Int]): Option[Map[CompInstanceIdent, CompAlternative]] = {
    RObj match {
      case Some(rVal) => cardObj match {
        case Some(cardVal) => resolvePb(List(
          (model) => isSafeCard(cardVal, model),
          (model) => isSafeUnr(1 - rVal, model)
        ))
        case None => resolvePb(List((model) => isSafeUnr(1 - rVal, model)))
      }
      case None => cardObj match {
        case Some(cardVal) => resolvePb(List((model) => isSafeCard(cardVal, model)))
        case None => None
      }
    }
  }

  def explore(RObj: Option[Real], cardObj: Option[Int]): Option[Map[CompInstanceIdent, CompAlternative]] = {
    RObj match {
      case Some(rVal) => cardObj match {
        case Some(cardVal) => resolvePb(List(
          (model) => isSafeCard(cardVal, model),
          (model) => isSafeUnrQuickXplain(1 - rVal, model)
        ))
        case None => resolvePb(List((model) => isSafeUnrQuickXplain(1 - rVal, model)))
      }
      case None => cardObj match {
        case Some(cardVal) => resolvePb(List((model) => isSafeCard(cardVal, model)))
        case None => None
      }
    }
  }

  def exploreSimple(RObj: Option[Real], cardObj: Option[Int]): Option[Map[CompInstanceIdent, CompAlternative]] = {
    RObj match {
      case Some(rVal) => cardObj match {
        case Some(cardVal) => resolvePb(List(
          (model) => isSafeCard(cardVal, model),
          (model) => isSafeUnrSimple(1 - rVal, model)
        ))
        case None => resolvePb(List((model) => isSafeUnrSimple(1 - rVal, model)))
      }
      case None => cardObj match {
        case Some(cardVal) => resolvePb(List((model) => isSafeCard(cardVal, model)))
        case None => None
      }
    }
  }

  protected def generateDecisionVars(): List[SubstitutionVar]

  protected def resolvePb(theoriesFunc: List[(Seq[(SubstitutionVar, Expr)]) => (Status, Seq[Seq[BoolExpr]])]): Option[Map[CompInstanceIdent, CompAlternative]] = {
    val s = _SMTContext.mkSolver()
    var assertionStack = KeepStack(new Z3SolverProxy(s))

    def buildReplaceMap(model: Map[SubstitutionVar, Expr]): Map[CompInstanceIdent, CompAlternative] = {
      model.collect({
        case (myVar, value) if _SMTContext.getValuePos(myVar, value).isDefined =>
          val sub = conf.designSpace.find(compSub => compSub.instance == myVar.inst).get
          val allAlts = sub.init +: sub.alternatives
          myVar.inst -> allAlts(_SMTContext.getValuePos(myVar, value).get)
      })
    }

    def addConflict(conflict: Seq[Seq[BoolExpr]]) = {
      if (assertionStack.containsNotKeep) {
        val jumpIndex = assertionStack.levelWhere(l => l.exists(e => conflict.flatten.contains(e._1)))
        assertionStack = assertionStack.pop(jumpIndex + 1)
      }
      assertionStack ::= (_SMTContext.mkNot(_SMTContext.mkAnd(conflict.map(c => _SMTContext.mkOr(c: _*)): _*)), true)
    }

    def addPropagation(model: Traversable[(SubstitutionVar, Expr)], cube: Seq[Seq[BoolExpr]]) = {
      if (cube.nonEmpty) {
        assertionStack ::= (_SMTContext.mkImplies(_SMTContext.toCube(model),
          _SMTContext.mkAnd(cube.map(c => _SMTContext.mkAnd(c.map(_SMTContext.mkNot): _*)): _*)), true)
      }
    }

    def mkDecision(model: Traversable[(SubstitutionVar, Expr)]) = {
      for (slct <- model.find(e => !e._1.sort.getConsts.contains(e._2))) //if a variable is not in the model
        yield {
          //add an temporary assertion which give an arbitrary value to the variable
          assertionStack = assertionStack.push
          assertionStack ::= _SMTContext.mkEq(slct._1.z3Var, slct._1.sort.getConst(0))
        }
    }

    val selectVars = generateDecisionVars()

    var status = Status.UNKNOWN
    var result: Option[Map[CompInstanceIdent, CompAlternative]] = None
    var archiExplored = 0
    //tant que le modele a des decisions possible ou qu il est satisfiable et qu'on n'a pas trouve
    while (status == Status.UNKNOWN) {
      archiExplored = archiExplored + 1
      val res = s.check()
      res match {
        case Status.SATISFIABLE =>
          val model = selectVars.map(e => (e, s.getModel.eval(e.z3Var, true)))
          val theoriesResults = theoriesFunc.map(fun => fun(model)).groupBy(_._1).mapValues(l => l.map(p => p._2))
          theoriesResults.get(Status.UNSATISFIABLE) match {
            case Some(clauses) => clauses.foreach(addConflict)
            case None =>
              theoriesResults.get(Status.UNKNOWN) match {
                case Some(props) =>
                  if (props.exists(cube => cube.nonEmpty)) {
                    props.filter(cube => cube.nonEmpty).foreach(cube => addPropagation(model, cube))
                  } else {
                    mkDecision(model)
                  }
                case None =>
                  status = Status.SATISFIABLE
                  result = Some(buildReplaceMap(model.toMap))
              }
          }
        case Status.UNSATISFIABLE =>
          if (assertionStack.containsNotKeep)
            assertionStack = assertionStack.reset
          else
            status = Status.UNSATISFIABLE

        case Status.UNKNOWN => throw Z3UnknownException(this)
      }
    }
    s.dispose()
    result
  }
}

trait CardTheoryDecorator extends CardinalityTheory {
  self: Parsing with SMTTranslation with STMDDBuilder =>
  val outputLogCard = new FileWriter(FileManager.addFileToTemp("cardLog.txt"))

  override def isSafeCard(obj: Int, model: Traversable[(SubstitutionVar, Expr)]): (Status, Seq[Seq[BoolExpr]]) = {
    val start = System.nanoTime()
    val res = super.isSafeCard(obj, model)
    val end = System.nanoTime()
    val r = "[|]".r
    outputLogCard.write({
      end - start
    } + "->" + res._1 + "," + r.replaceAllIn(res._2.mkString(","), "") + "\n")
    outputLogCard.flush()
    res
  }
}

trait CardTheoryClauseAnalyser extends CardinalityTheory {
  self: Parsing with SMTTranslation with STMDDBuilder =>
  val postProcess: ((Status, Seq[Seq[BoolExpr]])) => Unit

  override def isSafeCard(obj: Int, model: Traversable[(SubstitutionVar, Expr)]): (Status, Seq[Seq[BoolExpr]]) = {
    val res = super.isSafeCard(obj, model)
    postProcess(res)
    res
  }
}

trait UnrTheoryDecorator extends UnreliabilityTheory {
  self: Parsing with SMTTranslation with STMDDBuilder =>
  val outputLogUnr = new FileWriter(FileManager.addFileToTemp("unrLog.txt"))

  override def isSafeUnr(obj: Real, choice: Traversable[(SubstitutionVar, Expr)]): (Status, Seq[Seq[BoolExpr]]) = {
    val start = System.nanoTime()
    val res = super.isSafeUnr(obj, choice)
    val end = System.nanoTime()
    val r = "[|]".r
    outputLogUnr.write({
      end - start
    } + "->" + res._1 + "," + r.replaceAllIn(res._2.mkString(","), "") + "\n")
    outputLogUnr.flush()
    res
  }
}

trait UnrTheoryClauseAnalyser extends UnreliabilityTheory {
  self: Parsing with SMTTranslation with STMDDBuilder =>
  val postProcess: ((Status, Seq[Seq[BoolExpr]])) => Unit

  override def isSafeUnrQuickXplain(obj: Real, model: Traversable[(SubstitutionVar, Expr)]): (Status, Seq[Seq[BoolExpr]]) = {
    val res = super.isSafeUnrQuickXplain(obj, model)
    postProcess(res)
    res
  }
}

trait DesignSpaceExplorationKCRSim extends DesignSpaceExplorationKCR with ConfigurationAnalyzer with CardinalityTheorySim with UnreliabilityTheorySim {
  self: Parsing with SMTTranslation with SubstitutionAnalyzerSim with STMDDBuilderSim with TraceInterpreter =>

  def generateDecisionVars(): List[SubstitutionVar] = {
    val g = conf.namespace.locate(conf.rootCompId).namespace.compInstanceDepGraph
    g.inOrder.toList.collect({
      case n: g.Node if substitutions.contains(OrderedCompInstIdent(n.data)) =>
        val sub = conf.designSpace.find(s => s.instance == n.data).get
        val allAlts = sub.init +: sub.alternatives
        val sort = _SMTContext.mkEnumSort("EnumSort" + allAlts.size, allAlts.map(alt => alt.compId.baseId.name + alt.lambdaValues.map(p => p._2).mkString(",")): _*)
        val slctVar = _SMTContext.mkConst(n.data.toString, sort)
        SubstitutionVar(n.data, slctVar, sort)
    })
  }
}

trait DesignSpaceExplorationKCRSMT extends DesignSpaceExplorationKCR with ConfigurationAnalyzer with CardinalityTheorySMT with UnreliabilityTheorySMT {
  self: Parsing with SMTTranslation with SubstitutionAnalyzerSMT with STMDDBuilderSMT =>


  def generateDecisionVars(): List[SubstitutionVar] = {
    val g = conf.namespace.locate(conf.rootCompId).namespace.compInstanceDepGraph
    g.inOrder.toList.collect({
      case n: g.Node if conf.namespace.locate(conf.rootCompId).namespace.locateDef(n.data).nonEmpty =>
        substitutions.find(sub => sub.sub.instance.path.mkString(".") == n.data.path.mkString(".")) match {
          case Some(sub) =>
            val allAlts = sub.sub.init +: sub.sub.alternatives
            val sort = _SMTContext.mkEnumSort("EnumSort" + allAlts.size, allAlts.map(alt => alt.compId.baseId.name + alt.lambdaValues.map(p => p._2).mkString(",")): _*)
            val slctVar = _SMTContext.mkConst(sub.sub.instance.path.mkString("."), sort)
            SubstitutionVar(n.data, slctVar, sort)
          case None => //if no subsitution possible then create a set with only one possible value None
            val sort = _SMTContext.mkEnumSort("EnumSort1", "initial")
            val slctVar = _SMTContext.mkConst(n.data.path.mkString("."), sort)
            SubstitutionVar(n.data, slctVar, sort)
        }
    })
  }
}
