package old

import util.sat.FormulaLike

import scala.collection.immutable._
import scala.collection.{concurrent, mutable}

/**
  * Created by kdelmas on 10/06/16.
  */
class MyPrimeImplicantTranslator(cnfFormula: FormulaLike, target: FormulaLike, freeSet: Set[Int], switchMap: Map[Int, Int]) {
  val iniToTargetRenamingMap: mutable.Map[Int, Int] = new concurrent.TrieMap[Int, Int]()
  private var _myClauseIndex: Map[Int, Set[target.Clause]] = HashMap.empty

  /** Removes x from the given clause. */
  final private
  def restrict(x: Int, clause: target.Clause): target.Clause = {
    val l = (clause.lits.toSet - x).toList
    target.Clause(l)
  }

  /** Generates additional constraints enforcing the bijection between source's prime implicants and target's models for literal x.
    * Must be called after the clause index has been build (by a prior call to renameClauses).
    * */
  final
  def genBijectionConstraints(x: Int, clauses: Set[target.Clause]): Unit = {
    // remove x from all clauses that contain it
    val restrictedClauses = clauses.map { clause => restrict(x, clause) }

    if (restrictedClauses.isEmpty) {
      //if no clauses linked to variable x=0
      this.target.addClause(-x)()
    } else if (restrictedClauses.exists(clause => clause.lits.isEmpty)) {
      //if a clause contain only x then x=1
      this.target.addClause(x)()
    } else if (restrictedClauses.size == 1) {
      val g = this.target.freshVar
      this.target.defineOr(g, restrictedClauses.head.lits)()
      // add a clause (x => ~c) to the target solver
      this.target.addClause(-x, -g)()
    }
    else {
      //otherwise x=> not(and(restrictedClause)) ie at least one clause is false if x is not chosen
      // add a gate g = Or(clause.lits) for each clause in the restricted clauses
      val restrictedGates: List[Int] = restrictedClauses.map {
        clause => {
          val g = this.target.freshVar
          this.target.defineOr(g, clause.lits)()
          g
        }
      }.toList

      // build a conjunction gate c = And(gi) from the restricted clause gates
      val c = this.target.freshVar
      this.target.defineAnd(c, restrictedGates)()
      // add a clause (x => ~c) to the target solver
      this.target.addClause(-x, -c)()
    }
  }

  private def renameClauses(): Unit = {
    //first translate event vars
    freeSet.filter(lit => cnfFormula.isFree(lit)).foreach(i => {
      val x_p = target.freshVar
      val not_x_p = target.freshVar
      iniToTargetRenamingMap.update(i, x_p)
      iniToTargetRenamingMap.update(-i, not_x_p)
      target.addClause(-x_p, -not_x_p)()
      _myClauseIndex = _myClauseIndex.updated(x_p, Set[target.Clause]())
      _myClauseIndex = _myClauseIndex.updated(not_x_p, Set[target.Clause]())
    })
    // then translate gate vars
    freeSet.foreach(i => {
      if (iniToTargetRenamingMap.get(i).isEmpty) {
        val x_p = target.freshVar
        val not_x_p = target.freshVar
        iniToTargetRenamingMap.update(i, x_p)
        iniToTargetRenamingMap.update(-i, not_x_p)
        target.addClause(-x_p, -not_x_p)()
        _myClauseIndex = _myClauseIndex.updated(x_p, Set[target.Clause]())
        _myClauseIndex = _myClauseIndex.updated(not_x_p, Set[target.Clause]())
      }
    })
    // then translate switch vars
    switchMap.foreach(kv => {
      val newVar = target.freshVar
      iniToTargetRenamingMap.update(kv._2, newVar)
      iniToTargetRenamingMap.update(-kv._2, -newVar)
    })
    //then build the map giving all the clauses involving a litteral lit
    //for gates vars only clauses which are not heir definition
    this.cnfFormula.clauses.foreach(x => {
      val (clause, switch) = x
      val renamedClause = clause.lits.flatMap(lit => iniToTargetRenamingMap.get(lit))
      clause.lits.filterNot(lit => switchMap.values.exists(_ == Math.abs(lit))).foreach(lit => {
        val newLit = iniToTargetRenamingMap(lit)
        val absLit = Math.abs(lit)
        val clauses = cnfFormula.gates.get(lit) match {
          case None =>
            val newSet = _myClauseIndex.getOrElse(newLit, Set[target.Clause]()) + target.Clause(renamedClause)
            _myClauseIndex = _myClauseIndex.updated(newLit, newSet)
          case Some((g@cnfFormula.And(l: List[Int]), switchOpt)) =>
            val defs = cnfFormula.Clause(-switchMap(absLit) :: absLit :: l.map(e => -e)) +: l.map(e => cnfFormula.Clause(-switchMap(absLit) :: -absLit :: e :: Nil))
            if (!defs.contains(clause)) {
              val newSet = _myClauseIndex.getOrElse(newLit, Set[target.Clause]()) + target.Clause(renamedClause)
              _myClauseIndex = _myClauseIndex.updated(newLit, newSet)
            }
          case Some((g@cnfFormula.Or(l: List[Int]), switchOpt)) =>
            val defs = cnfFormula.Clause(-switchMap(absLit) :: -absLit :: l) +: l.map(e => cnfFormula.Clause(-switchMap(absLit) :: absLit :: -e :: Nil))
            if (!defs.contains(clause)) {
              val newSet = _myClauseIndex.getOrElse(newLit, Set[target.Clause]()) + target.Clause(renamedClause)
              _myClauseIndex = _myClauseIndex.updated(newLit, newSet)
            }
          case _ => ???
        }
      })
      target.addClauseList(renamedClause)()
    })

  }

  private def generatePIConstraints(): Unit = {
    _myClauseIndex foreach (kv => genBijectionConstraints(kv._1, kv._2))
    generateSwitchConstraints()
  }

  private def generateSwitchConstraints(): Unit = {
    switchMap.foreach(kv => {
      val x_p = iniToTargetRenamingMap(kv._1)
      val x_not_p = iniToTargetRenamingMap(-kv._1)
      val swicth = iniToTargetRenamingMap(kv._2)
      cnfFormula.gates.get(kv._1) match {
        case None => throw new Exception(s"${kv._1} should be a gate")
        case Some((g@cnfFormula.And(l: List[Int]), switchOpt)) =>
          val iniL = l.filterNot(_ == kv._2)
          val orGate = target.freshVar
          target.defineOr(orGate, iniL.map(lit => -iniToTargetRenamingMap(lit)))()
          target.defineAnd(-swicth, -x_p :: -x_not_p :: orGate :: iniL.map(lit => -iniToTargetRenamingMap(-lit)))()

        case Some((g@cnfFormula.Or(l: List[Int]), swithOpt)) =>
          val iniL = l.filterNot(_ == kv._2)
          val orGate = target.freshVar
          target.defineOr(orGate, iniL.map(lit => -iniToTargetRenamingMap(-lit)))()
          target.defineAnd(-swicth, -x_p :: -x_not_p :: orGate :: iniL.map(lit => -iniToTargetRenamingMap(lit)))()
        case _ => ???
      }


    })
  }

  private def renameCard(): Unit = {
    cnfFormula.cards.foreach(x => {
      val (card, switchOpt) = x
      card.relOp match {
        case this.cnfFormula.AtMost => target.addAtMostK(card.lits.map(iniToTargetRenamingMap), card.k)()
        case this.cnfFormula.AtLeast => target.addAtLeastK(card.lits.map(iniToTargetRenamingMap), card.k)()
        case this.cnfFormula.Exactly => target.addExactlyK(card.lits.map(iniToTargetRenamingMap), card.k)()
      }
    })
  }

  def translate() = {
    renameClauses()
    generatePIConstraints()
    renameCard()
  }

}
