/** $Id: ParsingUtils.scala 1433 2017-10-19 14:06:15Z kdelmas $
  *
  * @author $Author: kdelmas $
  **/

package parsers.dsl

import ast.dsl._

case class UnsupportedFeature(s: String) extends Exception {
  override def toString = s"Language feature '$s' is not yet supported."
}

/** Utility functions used when parsing identifiers. */
object ParsingUtils {

  /** Creates a type identifier from a list of strings. head of the list is the type name, tail is a package path. */
  def pkgTypeIdent(l: List[DSLSymbol]): TypeIdent = l match {
    case head :: tail => TypeIdent(head, PackageIdent(tail))
    case _ => throw new Exception("empty type identifier")
  }

  /** Creates a component identifier from a list of strings. head of the list is the component name, tail is a package path. */
  def pkgCompIdent(l: List[DSLSymbol]): CompIdent = l match {
    case head :: tail => CompIdent(head, PackageIdent(tail))
    case _ => throw new Exception("empty component identifier")
  }

  /** Creates a flow identifier from a list of strings. head of the list is the flow name, tail is a instance path relative to some call tree. */
  def instFlowIdent(l: List[DSLSymbol]): InstFlowIdent = l match {
    case head :: tail => InstFlowIdent(head, CompInstanceIdent(tail))
    case _ => throw new Exception("empty flow identifier")
  }

}
