/** $Id: Precedence.scala 1433 2017-10-19 14:06:15Z kdelmas $
  *
  * @author $Author: kdelmas $
  **/

package printing.dsl

import ast.dsl._

/** In this class each language operator in the expression language is
  * assigned an absolute precedence value (operators are globally
  * ordered), and offers a method to determine when a sub expression
  * must be parenthesised to preserve the correct AST structure when
  * printing an expression as text.
  */
object Precedence {

  /** Gives absolute precedence level for a boolean operator. */
  def apply(op: BoolExpr): Int = op match {
    case _: BoolIdent => 1800
    case _: FlowEq => 1000
    case _: FlowNeq => 1000
    case _: FlowContains => 1000
    case _: Not => 900
    case _: And => 800
    case _: Or => 700
    case _: Implies => 500
    case _: Iff => 400
    case _: PopCount => 300
  }

  def apply(e: FlowExpr): Int = e match {
    case _: FlowIdent => 1800
    case _: FlowUnion => 1700
    case _: FlowInter => 1600
    case _: FlowIte => 1500
  }

  /** True iff the kid must be parenthesised for printing. */
  def mustParen(parent: FlowExpr, kid: FlowExpr): Boolean = apply(kid) < apply(parent)

  /** True iff the kid must be parenthesised for printing. */
  def mustParen(parent: BoolExpr, kid: BoolExpr): Boolean = apply(kid) < apply(parent)

  /** True iff the kid must be parenthesised for printing. */
  def mustParen(parent: BoolExpr, kid: FlowExpr): Boolean = apply(kid) < apply(parent)

  /** True iff the kid must be parenthesised for printing. */
  def mustParen(parent: FlowExpr, kid: BoolExpr): Boolean = apply(kid) < apply(parent)

}

