package translators

import java.io.File

import ast.dsl._

import scala.collection.mutable
import scala.xml.{Elem, XML}

/**
  * Created by kdelmas on 10/02/17.
  */
object DSLToHipHOPSTranslator {
  val failure = "failure"
  val True = "true"
  val False = "false"
  val hipHopsDot = "_"
  val hipHopsHyphen = "#"
  val hipHopUnderscore = "$"
  val hipHopsEqual = "!"

  trait HipHOPSElement {
    val xml: Elem
  }

  trait HipHopsComp extends HipHOPSElement {
    val interface: Interface
    val name: String
  }

  trait HipHopsLeafComp extends HipHopsComp

  object HipHopsComp {
    var compCpt = 0
  }

  case class Interface(in: List[(BasicFlowIdent, String)], out: List[(BasicFlowIdent, String)]) extends HipHOPSElement {
    val xml: Elem =
      <Ports>
        {in.map(f => {
        <Port>
          <Type>Inport</Type>
          <Name>
            {f._2}
          </Name>
        </Port>
      })}{out.map(f => {
        <Port>
          <Type>Outport</Type>
          <Name>
            {f._2}
          </Name>
        </Port>
      })}
      </Ports>
  }

  case class HipHopsAlternativeComp(name: String,
                                    defs: Map[(BasicFlowIdent, TypeValueFlowIdent), String],
                                    evts: Map[BoolIdent, String],
                                    failureRate: Map[BoolIdent, BigDecimal],
                                    isOutComp: Boolean) extends HipHOPSElement {
    val xml: Elem =
      <Alternative>
        <Name>
          {name}
        </Name>
        <ExcludeFromOptimisation>false</ExcludeFromOptimisation>
        <Cost>0</Cost>
        <Weight>0</Weight>
        <HBlockType>Both</HBlockType>
        <FailureData>
          <ComponentFailureRate>0</ComponentFailureRate>
          <BasicEvents>
            {evts.map(e => {
            <BasicEvent>
              <Name>
                {e._2}
              </Name>
              <UnavailabilityFormula>
                <Constant>
                  <FailureRate>
                    {failureRate(e._1)}
                  </FailureRate>
                  <RepairRate>0</RepairRate>
                </Constant>
              </UnavailabilityFormula>
            </BasicEvent>
          })}
          </BasicEvents>
          <PotentialCCFs></PotentialCCFs>
          <OutputDeviations>
            {defs.map(p => {
            <OutputDeviation>
              <Name>
                {basicFlowIdentValueId(p._1._1, p._1._2)}
              </Name>
              <Severity>0</Severity>
              <SystemOutport>
                {isOutComp}
              </SystemOutport>
              <Causes>
                <Cause>
                  <Probability/>
                  <FailureLogic>
                    {p._2}
                  </FailureLogic>
                </Cause>
              </Causes>
            </OutputDeviation>
          })}
          </OutputDeviations>
          <ExportedPropagations></ExportedPropagations>
        </FailureData>
        <System>
          <Name>
            {name}
          </Name>
          <Components></Components>
        </System>
      </Alternative>
  }

  case class HipHopsPrimaryComp(interface: Interface,
                                name: String,
                                defs: Map[(BasicFlowIdent, TypeValueFlowIdent), String],
                                evts: Map[BoolIdent, String],
                                failureRate: Map[BoolIdent, BigDecimal],
                                isOutComp: Boolean,
                                alternatives: List[HipHopsAlternativeComp],
                                init: HipHopsAlternativeComp) extends HipHopsLeafComp {
    HipHopsComp.compCpt += 1
    val xml: Elem =
      <Component>
        <Type>SubSystem</Type>
        <Name>
          {name}
        </Name>
        <SID>component_
          {HipHopsComp.compCpt}
        </SID>
        <RiskTime>0</RiskTime>
        <IncludeInOptimisation>
          {alternatives.nonEmpty}
        </IncludeInOptimisation>{interface.xml}<Implementations>
        <Current>
          <Name>
            {init.name}
          </Name>
          <ExcludeFromOptimisation>false</ExcludeFromOptimisation>
          <Cost>0</Cost>
          <Weight>0</Weight>
          <HBlockType>Both</HBlockType>
          <FailureData>
            <ComponentFailureRate>0</ComponentFailureRate>
            <BasicEvents>
              {evts.map(e => {
              <BasicEvent>
                <Name>
                  {e._2}
                </Name>
                <UnavailabilityFormula>
                  <Constant>
                    <FailureRate>
                      {failureRate(e._1)}
                    </FailureRate>
                    <RepairRate>0</RepairRate>
                  </Constant>
                </UnavailabilityFormula>
              </BasicEvent>
            })}
            </BasicEvents>
            <PotentialCCFs></PotentialCCFs>
            <OutputDeviations>
              {defs.map(p => {
              <OutputDeviation>
                <Name>
                  {basicFlowIdentValueId(p._1._1, p._1._2)}
                </Name>
                <Severity>0</Severity>
                <SystemOutport>
                  {isOutComp}
                </SystemOutport>
                <Causes>
                  <Cause>
                    <Probability/>
                    <FailureLogic>
                      {p._2}
                    </FailureLogic>
                  </Cause>
                </Causes>
              </OutputDeviation>
            })}
            </OutputDeviations>
            <ExportedPropagations></ExportedPropagations>
          </FailureData>
          <System>
            <Name>
              {name}
            </Name>
            <Components></Components>
          </System>
        </Current>
        <Alternatives>
          {alternatives.map(_.xml)}
        </Alternatives>
      </Implementations>
      </Component>

  }

  case class ExpressionComp(interface: Interface,
                            name: String,
                            defs: Map[(BasicFlowIdent, TypeValueFlowIdent), String],
                            isOutComp: Boolean) extends HipHopsLeafComp {
    HipHopsComp.compCpt += 1
    val xml: Elem =
      <Component>
        <Type>SubSystem</Type>
        <Name>
          {name}
        </Name>
        <SID>component_
          {HipHopsComp.compCpt}
        </SID>
        <RiskTime>0</RiskTime>
        <IncludeInOptimisation>false</IncludeInOptimisation>{interface.xml}<Implementations>
        <Current>
          <Name>init</Name>
          <ExcludeFromOptimisation>false</ExcludeFromOptimisation>
          <Cost>0</Cost>
          <Weight>0</Weight>
          <HBlockType>Both</HBlockType>
          <FailureData>
            <ComponentFailureRate>0</ComponentFailureRate>
            <BasicEvents>
            </BasicEvents>
            <PotentialCCFs></PotentialCCFs>
            <OutputDeviations>
              {defs.map(p => {
              <OutputDeviation>
                <Name>
                  {basicFlowIdentValueId(p._1._1, p._1._2)}
                </Name>
                <Severity>0</Severity>
                <SystemOutport>
                  {isOutComp}
                </SystemOutport>
                <Causes>
                  <Cause>
                    <Probability/>
                    <FailureLogic>
                      {p._2}
                    </FailureLogic>
                  </Cause>
                </Causes>
              </OutputDeviation>
            })}
            </OutputDeviations>
            <ExportedPropagations></ExportedPropagations>
          </FailureData>
          <System>
            <Name>
              {name}
            </Name>
            <Components></Components>
          </System>
        </Current>
        <Alternatives>
        </Alternatives>
      </Implementations>
      </Component>
  }

  case class HipHopsSystemComp(interface: Interface,
                               name: String,
                               comps: List[HipHopsLeafComp],
                               connectors: List[Connector],
                               outMap: Map[BasicFlowIdent, String]) extends HipHopsComp {
    HipHopsComp.compCpt += 1
    val xml: Elem =
      <System>
        <Name>
          {name}
        </Name>
        <Components>
          {comps.map(_.xml)}
        </Components>
        <Lines>
          {connectors.map(_.xml)}
        </Lines>
      </System>
  }

  case class Connector(origin: Option[HipHopsComp], originPort: String, dest: Option[HipHopsComp], destPort: String) extends HipHOPSElement {
    val originS: String = origin match {
      case Some(c) => s"${c.name}.$originPort"
      case None => originPort
    }
    val destS: String = dest match {
      case Some(c) => s"${c.name}.$destPort"
      case None => destPort
    }
    val xml: Elem =
      <Line>
        <Name/> <Default>OR</Default>
        <Type>Directed</Type>
        <Connections>
          <Connection>
            <Port>
              {destS}
            </Port>
            <PortExpression>
              {originS}
            </PortExpression>
          </Connection>
        </Connections>
      </Line>
  }

  private def idOf(s: String) = {
    s.replace("|", "").replace("-", hipHopsHyphen).replace(".", hipHopsDot)
  }

  private def basicFlowIdentValueId(f: BasicFlowIdent, v: TypeValueFlowIdent, isFC: Boolean = false): String = {
    if (isFC)
      s"${idOf(v.valueId.name)}-${f.baseId.name}"
    else
      s"${idOf(v.valueId.name)}-${idOf(f.baseId.name)}"
  }

  private def flowIdentStringValueId(f: String, v: TypeValueFlowIdent, isFC: Boolean = false): String = {
    if (isFC)
      s"${idOf(v.valueId.name)}-$f"
    else
      s"${idOf(v.valueId.name)}-${idOf(f)}"
  }

  private def typeOf(c: ComponentDeclaration, b: FlowIdent) = {
    c.body.namespace.locate(b) match {
      case Left(tyype) => tyype
      case Right(fdecl) => c.parent.namespace.locate(fdecl.typeId)
    }
  }

  private def lambdaOf(b: BoolIdent, ini: CompIdent, i: CompInstanceIdent, c: ConfigurationDeclaration): BigDecimal = {
    {
      for (
        s <- c.designSpace.find(sub => sub.instance == i);
        a <- s.alternatives.find(_.compId == ini);
        l <- a.lambdaValues.find(_._1 == b)
      ) yield {
        BigDecimal(l._2.repr)
      }
    } getOrElse BigDecimal(0.1)
  }


  def mkAnd(b1: String, b2: String): String = {
    b1 match {
      case True => b2
      case False => False
      case v1 =>
        b2 match {
          case True => v1
          case False => False
          case v2 if v2.compareTo(v1) == 0 => v2
          case v2 => s"($v1 and $v2)"
        }
    }
  }

  def mkOr(b1: String, b2: String): String = {
    b1 match {
      case True => True
      case False => b2
      case v1 =>
        b2 match {
          case True => True
          case False => v1
          case v2 if v2.compareTo(v1) == 0 => v2
          case v2 => s"($v1 or $v2)"
        }
    }
  }

  def mkNot(b: String): String = {
    b match {
      case True => False
      case False => True
      case x =>
        val p = "\\(not (.*)\\)".r
        if (x.matches("\\(not .*\\)")) {
          val p(r) = x
          r
        } else {
          s"(not $x)"
        }
    }
  }


  def translateConfiguration(p: ConfigurationDeclaration, to: File, reliabilityReq: Option[String] = None): Unit = {
    val time = {
      for (t <- p.duration) yield t.repr
    } getOrElse "1"
    val upperBound = {
      for (r <- reliabilityReq) yield (1 - BigDecimal(r)).toString()
    } getOrElse {
      for (r <- p.reliability) yield (1 - BigDecimal(r.repr)).toString()
    }.getOrElse {
      "1.0"
    }
    val rootComp = translateRootComp(p.namespace.locate(p.rootCompId), p)
    val fc = translateBoolExpr(p.failureCondition, p.namespace.locate(p.rootCompId), Some(rootComp.outMap), None)
    val res =
      <Model>
        <Name>
          {idOf(to.getName.split("\\.").head)}
        </Name>
        <Version>1.9</Version>
        <ToolVersion></ToolVersion>
        <RiskTime>
          {time}
        </RiskTime>
        <OptimisationParams>
          <Objectives>
            <Objective>
              <Name>
                {s"Unavailability($failure)"}
              </Name>
              <Goal>Minimise</Goal>
              <Bounds>
                <UpperBound>
                  {upperBound}
                </UpperBound>
              </Bounds>
            </Objective>
          </Objectives>
        </OptimisationParams>
        <Hazards>
          <Hazard>
            <Name>Hazard1</Name>
            <Description/>
            <SafetyRequirement>0</SafetyRequirement>
            <Causes>
              <Cause>
                <Probability/>
                <FailureLogic>
                  {fc}
                </FailureLogic>
              </Cause>
            </Causes>
          </Hazard>
        </Hazards>
        <Perspectives>
          <Perspective>
            <Name>Default</Name>{rootComp.xml}
          </Perspective>
        </Perspectives>
      </Model>
    XML.save(to.getAbsolutePath, res, xmlDecl = true)
  }

  def rename(f: FlowExpr, renameMap: Map[BasicFlowIdent, BasicFlowIdent]): FlowExpr = {
    f match {
      case FlowIte(i, t, e) =>
        FlowIte(rename(i, renameMap), rename(t, renameMap), rename(e, renameMap))

      case FlowInter(f1, f2) =>
        FlowInter(rename(f1, renameMap), rename(f2, renameMap))

      case FlowUnion(f1, f2) =>
        FlowUnion(rename(f1, renameMap), rename(f2, renameMap))

      case id: BasicFlowIdent =>
        renameMap.getOrElse(id, id)

      case id: InstFlowIdent =>
        val bId = BasicFlowIdent(id.baseId)
        renameMap.getOrElse(bId, bId)

      case v: TypeValueFlowIdent => v
      case empty: TypeValueFlowEmpty => empty
    }
  }

  def rename(b: BoolExpr, renameMap: Map[BasicFlowIdent, BasicFlowIdent]): BoolExpr = b match {

      case And(b1, b2) =>
        And(rename(b1, renameMap), rename(b2, renameMap))

      case Or(b1, b2) =>
        Or(rename(b1, renameMap), rename(b2, renameMap))

      case Not(b1) =>
        Not(rename(b1, renameMap))

      case p@PopCount(kids, _, _) =>
        p.copy(kids = kids.map(x => rename(x, renameMap)))

      case FlowEq(f1, f2) =>
        FlowEq(rename(f1, renameMap), rename(f2, renameMap))

      case FlowNeq(f1, f2) =>
        FlowNeq(rename(f1, renameMap), rename(f2, renameMap))

      case FlowContains(f1, f2) =>
        FlowContains(rename(f1, renameMap), rename(f2, renameMap))

      case e: BoolIdent => e

      case _ => throw  ExprNotHandledByTranslation(b,"HipHOPS")
  }

  def flowIn(f: FlowExpr): Set[BasicFlowIdent] = {
    f match {
      case FlowIte(i, t, e) =>
        flowIn(i) ++ flowIn(t) ++ flowIn(e)

      case FlowInter(f1, f2) =>
        flowIn(f1) ++ flowIn(f2)

      case FlowUnion(f1, f2) =>
        flowIn(f1) ++ flowIn(f2)

      case id: BasicFlowIdent => Set(id)

      case id: InstFlowIdent => Set(BasicFlowIdent(id.baseId))

      case _ => Set()
    }
  }

  def flowIn(b: BoolExpr): Set[BasicFlowIdent] = {
    b match {
      case And(b1, b2) =>
        flowIn(b1) ++ flowIn(b2)

      case Or(b1, b2) =>
        flowIn(b1) ++ flowIn(b2)

      case Not(b1) =>
        flowIn(b1)

      case PopCount(kids, _, _) =>
        kids.flatMap(x => flowIn(x)).toSet

      case FlowEq(f1, f2) =>
        flowIn(f1) ++ flowIn(f2)

      case FlowNeq(f1, f2) =>
        flowIn(f1) ++ flowIn(f2)

      case FlowContains(f1, f2) =>
        flowIn(f1) ++ flowIn(f2)

      case _ => Set()
    }
  }

  def translateExpr(c: ComponentDeclaration, f: FlowExpr, isOutput: Boolean): ExpressionComp = {
    val out = BasicFlowIdent(StdDSLSymbol('out))
    val interface = Interface(flowIn(f).map(f => f -> idOf(f.baseId.name)).toList, List((out, "out")))
    val defs = translateFlowExpr(f, c, None, None).map(kv => (out, kv._1) -> kv._2)
    ExpressionComp(interface, s"component_${HipHopsComp.compCpt}", defs, isOutput)
  }

  def translateRootComp(c: ComponentDeclaration, conf: ConfigurationDeclaration): HipHopsSystemComp = {

    val defsInOrder = c.namespace.flowDepGraph.compDefsInOrder.toSeq
    val equiv = defsInOrder.foldLeft(Map[BasicFlowIdent, mutable.HashSet[BasicFlowIdent]]())((acc, fdef) => {
      fdef match {
        case CompCallFlowDefinition(lhs, _) =>
          acc ++ lhs.map(id => id -> mutable.HashSet[BasicFlowIdent]())
        case SingleFlowDefinition(lhs, rhs: BasicFlowIdent) =>
          for (kv <- acc.find(kv => (kv._2 + kv._1).contains(rhs))) yield {
            kv._2 += lhs
          }
          acc
        case SingleFlowDefinition(lhs, _) =>
          acc + (lhs -> mutable.HashSet())
      }
    })

    val renameMap = equiv.flatMap(kv => kv._2.map(e => e -> kv._1))

    val newDefs = c.body.definitions.collect({
      case CompCallFlowDefinition(lhs, rhs) =>
        CompCallFlowDefinition(lhs, rhs.copy(args = rhs.args.map(f => rename(f, renameMap))))
      case s@SingleFlowDefinition(lhs, rhs) if renameMap.get(lhs).isEmpty =>
        s.copy(rhs = rename(rhs, renameMap))
    })


    val flows = newDefs.collect({
      case CompCallFlowDefinition(lhs, rhs) =>
        val comp =
          translatePrimaryComp(
            c.parent.namespace.locate(rhs.compId),
            CompInstanceIdent(rhs.instId :: Nil),
            conf,
            {
              lhs.exists(f => c.outputs.map(_.ident).contains(f))
            })

        val args = rhs.args.collect({
          case f: BasicFlowIdent => f
          case expr => throw  ExprNotHandledByTranslation(expr,"HipHOPS")
        })
        val mapping = Interface(args.zip(comp.interface.in.unzip._2), lhs.zip(comp.interface.out.unzip._2))
        (mapping, comp)
      case SingleFlowDefinition(lhs, rhs) =>
        val comp = translateExpr(c, rhs, c.outputs.map(_.ident).contains(lhs))
        val mapping = Interface(comp.interface.in, List(lhs).zip(comp.interface.out.unzip._2))
        (mapping, comp)
    })

    val connectors = flows.flatMap(p => {
      p._1.out.flatMap(fPort => {
        flows.flatMap(p2 => {
          p2._1.in.collect({
            case fPort2 if fPort._1 == fPort2._1 => Connector(Some(p._2), fPort._2, Some(p2._2), fPort2._2)
          })
        })
      })
    })

    val outConnectors = c.outputs.flatMap(fdef => {
      flows.collect({
        case p if p._1.out.exists(outPort => outPort._1 == fdef.ident) =>
          val outPort = p._1.out.find(outPort => outPort._1 == fdef.ident).get
          fdef.ident -> s"${idOf(p._2.name)}.${idOf(outPort._2)}"
      })
    }).toMap

    val interface = Interface(List[(BasicFlowIdent, String)](), c.outputs.map(fdef => fdef.ident -> idOf(fdef.ident.baseId.name)))
    HipHopsSystemComp(
      interface,
      idOf(c.ident.baseId.name),
      flows.map(_._2),
      connectors,
      outConnectors)
  }


  def translateSingleFlowDefinition(f: SingleFlowDefinition, c: ComponentDeclaration): Map[(BasicFlowIdent, TypeValueFlowIdent), String] = {
    val tyype = typeOf(c, f.lhs)
    val fTr = translateFlowExpr(f.rhs, c, None, Some(f.lhs))
    tyype.values.map(v => TypeValueFlowIdent(tyype.ident, v)).map(v => (f.lhs, v) -> fTr(v)).toMap
  }

  def findLocalExpr(c: ComponentDeclaration, l: BasicFlowIdent): Option[FlowExpr] = {
    (for (
      e <- c.body.definitions.collect({ case SingleFlowDefinition(rhs, lhs) if rhs == l => lhs }).headOption
    ) yield {
      e
    }) orElse {
      for (
        e <- c.body.definitions.collect({ case SingleFlowDefinition(rhs, lhs) if rhs == l => lhs }).headOption) yield {
        e
      }
    }
  }

  def translateFlowExpr(f: FlowExpr, c: ComponentDeclaration, isFC: Option[Map[BasicFlowIdent, String]], isDef: Option[BasicFlowIdent]): Map[TypeValueFlowIdent, String] = {
    f match {
      case FlowIte(i, t, e) =>
        val iS = translateBoolExpr(i, c, isFC, isDef)
        val eS = translateFlowExpr(e, c, isFC, isDef)
        val tS = translateFlowExpr(t, c, isFC, isDef)
        eS.map(p => p._1 -> mkOr(mkAnd(iS, tS(p._1)), mkAnd(mkNot(iS), p._2)))

      case FlowInter(f1, f2) =>
        val f1S = translateFlowExpr(f1, c, isFC, isDef)
        val f2S = translateFlowExpr(f2, c, isFC, isDef)
        f1S.map(p => p._1 -> mkAnd(p._2, f2S(p._1)))

      case FlowUnion(f1, f2) =>
        val f1S = translateFlowExpr(f1, c, isFC, isDef)
        val f2S = translateFlowExpr(f2, c, isFC, isDef)
        f1S.map(p => p._1 -> mkOr(p._2, f2S(p._1)))

      case id: BasicFlowIdent =>
        (for (e <- findLocalExpr(c, id)) yield {
          translateFlowExpr(e, c, isFC, isDef)
        }) getOrElse {
          val tyype = typeOf(c, id)
          isFC match {
            case Some(trMap) =>
              tyype.values.map(s => TypeValueFlowIdent(tyype.ident, s)).map(v => v -> flowIdentStringValueId(trMap(id), v, isFC = true)).toMap
            case None =>
              tyype.values.map(s => TypeValueFlowIdent(tyype.ident, s)).map(v => v -> basicFlowIdentValueId(id, v)).toMap
          }
        }

      case id: InstFlowIdent =>
        val bId = BasicFlowIdent(id.baseId)
        (for (e <- findLocalExpr(c, bId)) yield {
          translateFlowExpr(e, c, isFC, isDef)
        }) getOrElse {
          val tyype = typeOf(c, id)
          isFC match {
            case Some(trMap) =>
              tyype.values.map(s => TypeValueFlowIdent(tyype.ident, s)).map(v => v -> flowIdentStringValueId(trMap(bId), v, isFC = true)).toMap
            case None =>
              tyype.values.map(s => TypeValueFlowIdent(tyype.ident, s)).map(v => v -> basicFlowIdentValueId(bId, v)).toMap
          }
        }

      case v: TypeValueFlowIdent =>
        val tyype = typeOf(c, v)
        tyype.values.map(s => TypeValueFlowIdent(tyype.ident, s)).map(x => x -> {
          if (v == x) True else False
        }).toMap

      case empty: TypeValueFlowEmpty =>
        val tyype = typeOf(c, empty)
        tyype.values.map(s => TypeValueFlowIdent(tyype.ident, s)).map(v => v -> False).toMap


      case _ => throw  ExprNotHandledByTranslation(f,"HipHOPS")
    }
  }

  def translateBoolExpr(b: BoolExpr, c: ComponentDeclaration, isFC: Option[Map[BasicFlowIdent, String]], isDef: Option[BasicFlowIdent]): String = {
    b match {
      case And(b1, b2) =>
        val sB1 = translateBoolExpr(b1, c, isFC, isDef)
        val sB2 = translateBoolExpr(b2, c, isFC, isDef)
        mkAnd(sB1, sB2)

      case Or(b1, b2) =>
        val sB1 = translateBoolExpr(b1, c, isFC, isDef)
        val sB2 = translateBoolExpr(b2, c, isFC, isDef)
        mkOr(sB1, sB2)

      case Not(b1) =>
        mkNot(translateBoolExpr(b1, c, isFC, isDef))

      case p@PopCount(kids, op, cst) =>
        op match {
          case Ge =>
            //naive translation
            val kidTr = kids.map(e => translateBoolExpr(e, c, isFC, isDef))
            val subsets = kidTr.toSet.subsets(cst.repr.toInt)
            subsets.foldLeft(False)((acc, s) => {
              val sVal = s.foldLeft(True)((acc2, e) => {
                mkAnd(acc2, e)
              })
              mkOr(acc, sVal)
            })

          case Gt => translateBoolExpr(p.copy(relop = Ge, cst = DecIntLit((cst.repr.toInt + 1).toString)), c, isFC, isDef)
          case Le => translateBoolExpr(Not(p.copy(relop = Gt)), c, isFC, isDef)
          case Lt => translateBoolExpr(Not(p.copy(relop = Ge)), c, isFC, isDef)
          case Eq => translateBoolExpr(And(p.copy(relop = Le), p.copy(relop = Ge)), c, isFC, isDef)
        }

      case FlowEq(f1, f2) =>
        val f1S = translateFlowExpr(f1, c, isFC, isDef)
        val f2S = translateFlowExpr(f2, c, isFC, isDef)
        val eq = f1S.map(p => p._1 -> mkOr(mkAnd(p._2, f2S(p._1)), mkAnd(mkNot(p._2), mkNot(f2S(p._1))))).foldLeft(True)((acc, v) => mkAnd(acc, v._2))
        eq

      case FlowNeq(f1, f2) => translateBoolExpr(Not(FlowEq(f1, f2)), c, isFC, isDef)

      case FlowContains(f1, f2) =>
        val f1S = translateFlowExpr(f1, c, isFC, isDef)
        val f2S = translateFlowExpr(f2, c, isFC, isDef)
        val imp = f1S.map(p => p._1 -> mkOr(mkNot(p._2), f2S(p._1))).foldLeft(True)((acc, v) => mkAnd(acc, v._2))
        imp

      case e: BoolIdent =>
        if(isDef.nonEmpty){
          idOf(e.baseId.name)
        } else {
          throw ForbiddenInFC(e)
        }

      case _ => throw ExprNotHandledByTranslation(b,"HipHOPS")
    }
  }

  def translatePrimaryComp(c: ComponentDeclaration, inst: CompInstanceIdent, conf: ConfigurationDeclaration, isOutComp: Boolean): HipHopsPrimaryComp = {
    assert(c.isPrimary, "no primary components")
    val zero = Map[(BasicFlowIdent, TypeValueFlowIdent), String]()
    val flows = c.namespace.flowDepGraph.compDefsInOrder.toList.distinct.foldLeft(zero)((acc, fdef) => fdef match {
      //locals have been inlined
      case SingleFlowDefinition(rhs, _) if c.body.locals.exists(_.ident == rhs) => acc
      //there is no way to represents true or false with HipHOPS
      case fdef: SingleFlowDefinition => acc ++ translateSingleFlowDefinition(fdef, c).filter({
        case (_, True) => false
        case (_, False) => false
        case _ => true
      })
      case _ => acc
    })
    val interface = Interface(c.inputs.map(fdef => fdef.ident -> idOf(fdef.ident.baseId.name)),
      c.outputs.map(fdef => fdef.ident -> idOf(fdef.ident.baseId.name)))

    val alts = for (s <- conf.designSpace.find(sub => sub.instance == inst)) yield {
      s.alternatives.map(alt => translateAlternativeComp(alt, conf, isOutComp))
    }

    val init = for (s <- conf.designSpace.find(sub => sub.instance == inst)) yield {
      translateAlternativeComp(s.init, conf, isOutComp)
    }

    HipHopsPrimaryComp(
      interface,
      idOf(inst.path.last.name),
      flows,
      c.body.events.map(e => e -> idOf(e.baseId.name)).toMap,
      c.body.events.map(e => e -> lambdaOf(e, c.ident, inst, conf)).toMap,
      isOutComp,
      alts.getOrElse(Nil),
      init.get
    )
  }

  def translateAlternativeComp(alt: CompAlternative, conf: ConfigurationDeclaration, isOutComp: Boolean): HipHopsAlternativeComp = {
    val c = conf.namespace.locate(alt.compId)
    assert(c.isPrimary, "no primary components")
    val zero = Map[(BasicFlowIdent, TypeValueFlowIdent), String]()
    val flows = c.namespace.flowDepGraph.compDefsInOrder.toList.distinct.foldLeft(zero)((acc, fdef) => fdef match {
      //locals have been inlined
      case SingleFlowDefinition(rhs, _) if c.body.locals.exists(_.ident == rhs) => acc
      //there is no way to represents true or false with HipHOPS
      case fdef: SingleFlowDefinition => acc ++ translateSingleFlowDefinition(fdef, c).filter({
        case (_, True) => false
        case (_, False) => false
        case _ => true
      })
      case _ => acc
    })

    HipHopsAlternativeComp(
      nameOfAlt(alt),
      flows,
      c.body.events.map(e => e -> idOf(e.baseId.name)).toMap,
      alt.lambdaValues.toMap.mapValues(v => BigDecimal(v.repr)),
      isOutComp
    )
  }

  def nameOfAlt(alt: CompAlternative): String = {
    idOf(s"${alt.compId}$hipHopUnderscore${alt.lambdaValues.map(p => s"${p._1}$hipHopsEqual${p._2}").mkString(hipHopUnderscore)}")
  }

  def altFromString(inst: String, alt: String, conf: ConfigurationDeclaration): Option[(CompInstanceIdent, CompAlternative)] = {
    for (sub <- conf.designSpace.find(sub => idOf(sub.instance.path.last.name) == inst);
         a <- (sub.init +: sub.alternatives).find(e => nameOfAlt(e) == alt)) yield {
      (sub.instance, a)
    }
  }

}
