package ui

import java.awt._
import java.awt.event.{ActionEvent, ActionListener, MouseAdapter, MouseEvent}
import javax.swing._
import javax.swing.plaf.basic.BasicButtonUI

/**
  * Created by kdelmas on 16/01/17.
  */
class ButtonTabComponent(val title: String, val pane: JTabbedPane) extends JPanel {
  //unset default FlowLayout' gaps
  setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0))
  setOpaque(false)

  //make JLabel read titles from JTabbedPane
  val label = new JLabel(title)

  add(label)
  //add more space between the label and the button
  label.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5))
  //tab button
  val button = new TabButton()
  add(button)
  //add more space to the top of the component
  setBorder(BorderFactory.createEmptyBorder(2, 0, 0, 0))

  def setText(text: String): Unit = {
    label.setText(text)
  }

  def getText: String = {
    label.getText
  }

  class TabButton extends JButton with ActionListener {
    val mySize = 17
    setPreferredSize(new Dimension(mySize, mySize))
    setToolTipText("close this tab")
    //Make the button looks the same for all Laf's
    setUI(new BasicButtonUI())
    //Make it transparent
    setContentAreaFilled(false)
    //No need to be focusable
    setFocusable(false)
    setBorder(BorderFactory.createEtchedBorder())
    setBorderPainted(false)
    //Making nice rollover effect
    //we use the same listener for all buttons
    addMouseListener(TabCloseListener)
    setRolloverEnabled(true)
    //Close the proper tab by clicking the button
    addActionListener(this)

    def actionPerformed(e: ActionEvent) {
      val i = pane.indexOfTabComponent(ButtonTabComponent.this)
      if (i != -1) {
        pane.remove(i)
      }
    }

    //we don't want to update UI for this button
    override def updateUI() {}

    //paint the cross
    override protected def paintComponent(g: Graphics) {
      super.paintComponent(g)
      val g2 = g.create().asInstanceOf[Graphics2D]
      //shift the image for pressed buttons
      if (getModel.isPressed) {
        g2.translate(1, 1)
      }
      g2.setStroke(new BasicStroke(2))
      g2.setColor(Color.BLACK)
      if (getModel.isRollover) {
        g2.setColor(Color.RED)
      }
      val delta = 6
      g2.drawLine(delta, delta, getWidth - delta - 1, getHeight - delta - 1)
      g2.drawLine(getWidth - delta - 1, delta, delta, getHeight - delta - 1)
      g2.dispose()
    }
  }

}

object TabCloseListener extends MouseAdapter {
  override def mouseEntered(e: MouseEvent) {
    e.getComponent match {
      case component: AbstractButton =>
        val button = component.asInstanceOf[AbstractButton]
        button.setBorderPainted(true)
    }
  }


  override def mouseExited(e: MouseEvent) {
    e.getComponent match {
      case component: AbstractButton =>
        val button = component.asInstanceOf[AbstractButton]
        button.setBorderPainted(false)
    }
  }
}
