package ui

import java.awt.Color
import javax.swing.JTextPane
import javax.swing.text.{DefaultStyledDocument, Style, StyleConstants, StyledDocument}

import util.analyzers.Console

/**
  * Created by kdelmas on 17/01/17.
  */
class ConsoleTextPane(val doc: StyledDocument = new DefaultStyledDocument()) extends JTextPane(doc)  with Console {
  val infoStyle: Style = doc.addStyle("info", getLogicalStyle)
  StyleConstants.setForeground(infoStyle, new Color(0, 0, 153))

  val errorStyle: Style = doc.addStyle("error", getLogicalStyle)
  StyleConstants.setForeground(errorStyle, new Color(153, 0, 0))

  val resultStyle: Style = doc.addStyle("result", getLogicalStyle)
  StyleConstants.setForeground(resultStyle, new Color(0, 102, 0))

  val warningStyle: Style = doc.addStyle("warning", getLogicalStyle)
  StyleConstants.setForeground(warningStyle, new Color(255, 128, 0))

  def printInfo(s: String): Unit = {
    doc.insertString(doc.getLength, "[INFO] " + s + "\n", infoStyle)
  }

  def printWarning(s: String): Unit = {
    doc.insertString(doc.getLength, "[WARNING] " + s + "\n", warningStyle)
  }

  def printError(s: String): Unit = {
    doc.insertString(doc.getLength, "[ERROR] " + s + "\n", errorStyle)
  }

  def printResult(s: String): Unit = {
    doc.insertString(doc.getLength, "[RESULT] " + s + "\n", resultStyle)
  }

  def clear(): Unit = {
    doc.remove(0, doc.getLength)
  }
}
