package ui

import java.awt.{Color, Font}

/**
  * Created by kdelmas on 03/02/17.
  */

trait Diagram {
  def addElement(e: DiagramElement): Unit
}

trait DiagramView extends javax.swing.JComponent {
  def setDiagram(diagram: Diagram): Unit
}

sealed trait ColorStyle

case class PlainColor(color: Color) extends ColorStyle

case class GradientColor(start: Color, end: Color, angle: Int) extends ColorStyle

sealed trait Shape

case object Rectangle extends Shape

case object Circle extends Shape

case object Triangle extends Shape

sealed trait Direction

case object Horizontal extends Direction

case object Vertical extends Direction

trait DiagramElement

trait DiagramNode extends DiagramElement {
  val name: String

  def setBackground(colorStyle: ColorStyle): Unit

  def setForeground(colorStyle: ColorStyle): Unit

  def setFont(font: Font): Unit

  def fitToContent(): Unit
}

trait BasicNode extends DiagramNode {
  def setShape(shape: Shape): Unit
}

trait ContainerNode extends DiagramNode {
  def addChild(node: DiagramNode): Unit

  def removeChild(node: DiagramNode): Unit

  def addChild(childList: Traversable[DiagramNode]): Unit

  def removeAll(): Unit
}

trait DiagramLink extends DiagramElement {
  val origin: DiagramNode
  val destination: DiagramNode

  def setOriginShape(shape: Shape): Unit

  def setDestinationShape(shape: Shape): Unit

  def setOriginShapeColor(colorStyle: ColorStyle): Unit

  def setDestinationShapeColor(colorStyle: ColorStyle): Unit

  def setOriginShapeSize(size: Double): Unit

  def setDestinationShapeSize(size: Double): Unit

  def setLabel(label: String): Unit

  def getLabel: String
}

trait BasicDiagramLink extends DiagramLink

trait Layout {
  def arrangeDiagram(diagram: Diagram): Unit
}

trait DirectedLayout extends Layout {
  def setDirection(direction: Direction): Unit
}

trait TreeLayout extends Layout

trait LayerLayout extends DirectedLayout


trait DiagramDrawerFactory {
  def createBasicNode(name: String): BasicNode

  def createContainerNode(name: String): ContainerNode

  def createLink(diagram: Diagram, origin: DiagramNode, destination: DiagramNode): DiagramLink

  def createTreeLayout: TreeLayout

  def createLayerLayout(direction: Direction): LayerLayout

  def createDiagram: Diagram

  def createDiagramView: DiagramView
}