package ui

import com.mindfusion.diagramming

/**
  * Created by kdelmas on 03/02/17.
  */
object GSFactory extends DiagramDrawerFactory {

  trait GSElement extends diagramming.DiagramItem with DiagramElement

  class GSDiagram extends diagramming.Diagram with Diagram {
    def addElement(e: DiagramElement): Unit = {
      e match {
        case x: GSBasicNode => add(x)
        case x: GSContainerNode => add(x)
        case x: GSLink => add(x)
      }
    }
  }

  trait GSNode extends diagramming.DiagramNode with DiagramNode

  class GSDiagramView extends diagramming.DiagramView with DiagramView {
    def setDiagram(diagram: Diagram): Unit = setDiagram(diagram.asInstanceOf[diagramming.Diagram])

    setAllowInplaceEdit(false)
  }

  class GSBasicNode(val name: String) extends diagramming.ShapeNode with BasicNode with GSNode {

    setShadowOffsetX(0)
    setShadowOffsetY(0)
    setText(name)

    def setShape(shape: Shape): Unit = setShape(convert(shape))

    def setBackground(colorStyle: ColorStyle): Unit = setBrush(convert(colorStyle))

    def setForeground(colorStyle: ColorStyle): Unit = setTextBrush(convert(colorStyle))

    def fitToContent(): Unit = {
      if (getFont != null)
        resize(getFont.getSize * (getText.length + 2), getFont.getSize * 2)
    }
  }

  class GSContainerNode(val name: String) extends diagramming.ContainerNode with ContainerNode with GSNode {
    setShadowOffsetX(0)
    setShadowOffsetY(0)
    setEditedText(name)
    setFolded(true)

    def setBackground(colorStyle: ColorStyle): Unit = setBrush(convert(colorStyle))

    def setForeground(colorStyle: ColorStyle): Unit = setTextBrush(convert(colorStyle))

    def fitToContent(): Unit = updateBounds()

    def addChild(node: DiagramNode): Unit = add(convert(node))

    def removeChild(node: DiagramNode): Unit = remove(convert(node))

    def addChild(childList: Traversable[DiagramNode]): Unit = childList.foreach(n => add(convert(n)))

    def removeAll(): Unit = ???
  }

  class GSLink(val diagram: Diagram, val origin: DiagramNode, val destination: DiagramNode)
    extends diagramming.DiagramLink(convert(diagram), convert(origin), convert(destination)) with DiagramLink {

    setShadowOffsetX(0)
    setShadowOffsetY(0)

    def setOriginShape(shape: Shape): Unit = setBaseShape(convert(shape))

    def setDestinationShape(shape: Shape): Unit = setHeadShape(convert(shape))

    def setOriginShapeSize(size: Double): Unit = setBaseShapeSize(size.toFloat)

    def setDestinationShapeSize(size: Double): Unit = setHeadShapeSize(size.toFloat)

    def setOriginShapeColor(colorStyle: ColorStyle): Unit = setBaseBrush(convert(colorStyle))

    def setDestinationShapeColor(colorStyle: ColorStyle): Unit = setHeadBrush(convert(colorStyle))

    def setLabel(label: String): Unit = setText(label)

    def getLabel: String = getText
  }

  trait GSLayout extends diagramming.Layout with Layout {
    def arrangeDiagram(diagram: Diagram): Unit = arrange(convert(diagram))
  }

  class GSLayerLayout extends diagramming.LayeredLayout with LayerLayout with GSLayout {
    setDirection(Horizontal)
    setSplitLayers(true)
    setLayerDistance(20)
    setYGap(10)

    def setDirection(direction: Direction): Unit = setOrientation(convert(direction))

    def arrange(diagram: Diagram, elements: Set[DiagramElement]): Unit = {
      val d = convert(diagram)
      val l = new diagramming.DiagramItemList()
      elements.foreach({ case e: GSElement => l.add(e) })
      arrange(d, l)
    }
  }

  class GSTreeLayout extends diagramming.TreeLayout with TreeLayout with GSLayout

  def createBasicNode(name: String): BasicNode = new GSBasicNode(name)

  def createContainerNode(name: String): ContainerNode = new GSContainerNode(name)

  def createLink(diagram: Diagram, origin: DiagramNode, destination: DiagramNode): DiagramLink = new GSLink(diagram, origin, destination)

  def createTreeLayout: TreeLayout = new GSTreeLayout

  def createLayerLayout(direction: Direction = Horizontal): LayerLayout = {
    val r = new GSLayerLayout
    r.setDirection(direction)
    r
  }

  def createDiagram: Diagram = new GSDiagram

  def createDiagramView: DiagramView = new GSDiagramView

  def convert(x: ContainerNode): GSContainerNode = x.asInstanceOf[GSContainerNode]

  def convert(x: BasicNode): GSBasicNode = x.asInstanceOf[GSBasicNode]

  def convert(x: Diagram): GSDiagram = x.asInstanceOf[GSDiagram]

  def convert(x: DiagramView): GSDiagramView = x.asInstanceOf[GSDiagramView]

  def convert(x: BasicDiagramLink): GSLink = x.asInstanceOf[GSLink]

  def convert(x: DiagramNode): GSNode = {
    x match {
      case n: GSContainerNode => n
      case n: GSBasicNode => n
    }
  }

  def convert(x: Shape): diagramming.Shape = {
    x match {
      case Rectangle => diagramming.Shape.fromId("Rectangle")
      case Circle => diagramming.Shape.fromId("Circle")
      case Triangle => diagramming.Shape.fromId("Triangle")
    }
  }

  def convert(x: ColorStyle): diagramming.Brush = {
    x match {
      case PlainColor(c) => new diagramming.SolidBrush(c)
      case GradientColor(s, e, angle) => new diagramming.GradientBrush(s, e, angle)
    }
  }

  def convert(x: Direction): diagramming.jlayout.Orientation = {
    x match {
      case Horizontal => diagramming.jlayout.Orientation.Horizontal
      case Vertical => diagramming.jlayout.Orientation.Vertical
    }
  }
}




