package util.analyzers

import analyzers.Analyser
import ast.dsl.{CompIdent, CompInstanceIdent, DSLSymbol}

//base trait for analysis exceptions
trait AnalyzerExceptions extends Exception

case class Z3UnknownException(analysis: Analyser) extends AnalyzerExceptions {
  override def toString = s"Z3 solver could not solve SMT problem for analysis $analysis"
}

case class Z3UnsatException(analysis: Analyser) extends AnalyzerExceptions {
  override def toString = s"The SMT problem of $analysis is UNSAT but should be SAT"
}

case class IllegalSubstitution(compInst: CompInstanceIdent, sub: CompIdent, trace: String) extends AnalyzerExceptions {
  override def toString = s"$sub is not an acceptable substitution for $compInst since trace $trace is acceptable for $sub but not for $compInst"
}

case class MissingBaseAlternative(compInstanceIdent: DSLSymbol) extends AnalyzerExceptions {
  override def toString = s"The instance $compInstanceIdent must have at least one alternative in design space field"
}

case class MissingSubstitution(compInstanceIdent: CompInstanceIdent) extends AnalyzerExceptions {
  override def toString = s"Missing substitution in design space for instance $compInstanceIdent"
}

case class UnknownInstance(compInstanceIdent: CompInstanceIdent) extends AnalyzerExceptions {
  override def toString = s"Unknown instance $compInstanceIdent"
}

case class UnknownEvent(eventName: String) extends AnalyzerExceptions {
  override def toString = s"Unknown event $eventName"
}

case class WrongBoolExpr(expression: String) extends AnalyzerExceptions {
  override def toString = s"the expression $expression contains non Booleans connectives and/or variables"
}

case class WrongBDDSimplification(node: String) extends AnalyzerExceptions {
  override def toString = s"the node $node cannot belong to a simplified BDD"
}

case class UnknownZ3AST(node: String) extends AnalyzerExceptions {
  override def toString = s"the Z3 AST node $node cannot be processed for BDD build"
}

case class ParsingFailure(cause: String) extends AnalyzerExceptions {
  override def toString = s"failed to build analyser caused by \n $cause"
}
