package util.analyzers

import ast.dsl.BasicFlowIdent
import com.microsoft.z3.BitVecNum

/**
  * Created by kdelmas on 04/11/16.
  */

/**
  * Case class encoding a partial function over component traces
  * @param func the function from component trace to value
  * @param default the default value
  * @tparam T the type of return values
  */
case class CharacteristicFun[T](func: Map[Map[BasicFlowIdent, BitVecNum], T], default: T) {

  /**
    * Return the sum of the values given by the function to a partial component trace
    * @param trace possibly partial component trace
    * @param num values type is Numeric
    * @return sum of the values if exists, otherwise the default value
    */
  def getAggregation(trace: Map[BasicFlowIdent, BitVecNum])(implicit num: Numeric[T]): T = {
    func.filter(p => trace.forall(c => p._1.toList.contains(c))).toList match {
      case Nil => default
      case traces => traces.map(p => p._2).sum
    }
  }

  /**
    * Return the first value given by the function to a possibly partial component trace
    * @param trace possibly partial component trace
    * @return first value, otherwise the default value
    */
  def getFirst(trace: Map[BasicFlowIdent, BitVecNum]): T = {
    func.find(p => trace.forall(c => p._1.toList.contains(c))) match {
      case Some(x) => x._2
      case None => default
    }
  }
}
