package util.analyzers

import ast.dsl.{InstBoolIdent, InstFlowIdent}
import util.translators.SMTContext
import com.microsoft.z3.{Sort => Z3Sort,
  Symbol => Z3Symbol, FuncDecl => Z3FuncDecl,
  Solver => Z3Solver, BoolExpr => Z3BoolExpr,
  Expr => Z3Expr,
  BitVecExpr => Z3BitVecExpr,
  BitVecNum}
/**
  * class representing a component for component analysis by SMT-based approach
  *
  * @param inputs the map from component inputs to Z3 bitvector expressions
  * @param outputs the map of component outputs to Z3 bitvector expression
  * @param events the map of component events to Z3 boolean expression
  * @param context the SMT context of used to build SMT expressions
  */
class Component(val inputs: Map[InstFlowIdent, Z3BitVecExpr],
                val outputs: Map[InstFlowIdent, Z3BitVecExpr],
                val events: Map[InstBoolIdent, Z3BoolExpr],
                val context: SMTContext) {
  def this(comp: Component) = {
    this(comp.inputs, comp.outputs, comp.events, comp.context)
  }
}
