package util.analyzers

/**
  * Base trait for available cut computation modes
  */
trait CutComputationMode

/**
  * Cut computation based on Iterative SAT method
  */
object IterativeSATCutComputation extends CutComputationMode {
  override def toString: String = "Iterative"
}

/**
  * Cut computation based on Iterative SMT method
  */
object IterativeSMTCutComputation extends CutComputationMode {
  override def toString: String = "SMT"
}

/**
  * Cut computation based on BDD analysis
  */
object BDDCutComputation extends CutComputationMode {
  override def toString: String = "BDD"
}

/**
  * Cut enumeration based on Pseudo-Boolean solving
  */
object PBCutComputation extends CutComputationMode {
  override def toString: String = "Pseudo Boolean"
}

/**
  * Cut enumeration based on XFTA external tool
  */
object XFTACutComputation extends CutComputationMode {
  override def toString: String = "XFTA"
}

/**
  * Cut enumeration based on XSAP external tool
  */
object XSAPCutComputation extends CutComputationMode {
  override def toString: String = "xSAP"
}

/**
  * Cut enumeration based on HIPHOPS external tool
  */
object HipHOPSCutComputation extends CutComputationMode {
  override def toString: String = "HipHOPS"
}
