package util.analyzers

import analyzers._
import util.analyzers.analyzerTypes._
import ast.dsl.{BasicFlowIdent, InstBoolIdent, InstFlowIdent}
import com.microsoft.z3.{Sort => Z3Sort,
  Symbol => Z3Symbol, FuncDecl => Z3FuncDecl,
  Solver => Z3Solver, BoolExpr => Z3BoolExpr,
  Expr => Z3Expr,
  BitVecExpr => Z3BitVecExpr,
  BitVecNum}
import  com.microsoft.z3.Status._
import net.sf.javabdd.BDD

trait FBDDBuildSMT extends FBDDBuild {
  self: Component with RelationBuild =>
  val BDDFactory: BDDFactory with Z3BoolExprToBDDTranslator

  protected def computeFBDD(): (CharacteristicFun[BDD], Map[InstBoolIdent, BDD]) = {
    var f = Map[Z3TraceType, BDD]()
    val eventBDD: Iterable[(InstBoolIdent, Z3BoolExpr, BDD)] = relEvtField.map(kv => (kv._1, kv._2, BDDFactory.produceVar()))
    //uninterpreted function true is the component events valuation trigger a given trace
    val tempFunc: Z3FuncDecl = context.mkFuncDecl("tempFunc", events.map(e => e._2.getSort).toArray, context.mkBoolSort())
    val tempFuncApply: Z3Expr = tempFunc.apply(events.values.toList: _*)
    //a solver to enumerate all traces
    val inputGen: Z3Solver = context.mkSolver()
    //a solver to find the model of tempFunc
    val funcGen: Z3Solver = context.mkSolver()
    //all interface flows are defined as constants
    val traceConst: List[(InstFlowIdent, Z3BitVecExpr, Z3BitVecExpr)] =
      relFlowField.map(kv => (kv._1, kv._2, context.mkConst(kv._1.toString, kv._2.getSort).asInstanceOf[Z3BitVecExpr])).toList
    val constVars: (List[InstFlowIdent], List[Z3BitVecExpr], List[Z3BitVecExpr]) = traceConst.unzip3
    //flow constants replace the initial bounded variable in the relation expression
    val relSub: Z3BoolExpr = relExprField.substitute(constVars._2.toArray[Z3Expr], constVars._3.toArray[Z3Expr]).asInstanceOf[Z3BoolExpr]
    //creating the formula (forall e1..ek (tempFunc e1..ek)= ((c in1..inn e1..ek)=(o1..om))
    val evtForall: List[(Z3Sort, Z3Symbol)] = relEvtField.toList.sortBy(p => p._2.getIndex).map(kv => (kv._2.getSort, context.mkSymbol(kv._1.toString)))
    val evtUnZip: (List[Z3Sort], List[Z3Symbol]) = evtForall.unzip
    val formula: Z3BoolExpr = context.mkForall(
      evtUnZip._1.toArray,
      evtUnZip._2.toArray[Z3Symbol].reverse,
      context.mkEq(tempFuncApply, relSub),
      0, null, null, null, null)
    //and add the formula to the tempFunc solver
    funcGen.add(formula)

    //building the formula (c in1..inn e1..ek)=(o1..om) where variables are problem's constants
    val relConst: (Iterable[Z3Expr], Iterable[Z3Expr]) = (relFlowField ++ relEvtField).map(p => (p._2, context.mkConst(p._1.toString, p._2.getSort))).unzip
    //and add it to the trace enumeration solver
    inputGen.add(relExprField.substitute(relConst._1.toArray[Z3Expr], relConst._2.toArray[Z3Expr]).asInstanceOf[Z3BoolExpr])

    //save the initial state of the tempFunc solver
    funcGen.push()

    while (inputGen.check() == SATISFIABLE) {
      //while its exists a trace satisfying (c in1..inn e1..ek)=(o1..om) then add it to the tempFunc solver
      val mdl: List[(InstFlowIdent, Z3BitVecExpr, BitVecNum)] = traceConst.map(p => (p._1, p._3, inputGen.getModel.eval(p._3, true).asInstanceOf[BitVecNum]))
      funcGen.pop()
      funcGen.push()
      funcGen.add(context.mkAnd(mdl.map(p => context.mkEq(p._2, p._3)): _*))
      //and check to find a model of the tempFunc for the current trace
      funcGen.check() match {
        case SATISFIABLE =>
          // the model of the trace and the tempFunc are extracted and tempFunc is translated to a BDD
          val trace: List[(InstFlowIdent, BitVecNum)] = mdl.map(t => (t._1, t._3))
          val tempFuncVal: Z3BoolExpr = funcGen.getModel.getFuncInterp(tempFunc).getElse.asInstanceOf[Z3BoolExpr]
          val bdd: BDD = BDDFactory.boolExprToBdd(tempFuncVal, eventBDD.map(t => (t._2, t._3)).toMap, context)
          f = f + Tuple2(trace, bdd)
          //find a new trace by forbidding the current trace
          inputGen.add(context.mkNot(context.mkAnd(mdl.map(t => context.mkEq(t._2, t._3)): _*)))
        case UNKNOWN => throw Z3UnknownException(this)
        case UNSATISFIABLE => throw Z3UnsatException(this)
      }
    }
    if (inputGen.check() == UNKNOWN)
      throw Z3UnknownException(this)
    else {
      //when all traces have been found then close solvers and return the BDD function for each traces
      inputGen.dispose()
      funcGen.dispose()
      val zero = BDDFactory.zero()
      (CharacteristicFun[BDD](f.map(kv => kv._1.map(kv2 => BasicFlowIdent(kv2._1.baseId) -> kv2._2).toMap -> kv._2), zero),
        eventBDD.map(t => (t._1, t._3)).toMap)
    }
  }
}
