package util.analyzers

import util.analyzers.MDDType.UidType

object Node {

  /** Global node instance counter */
  final var uid: UidType = -1

  /** Returns the next available and unused uid. */
  def nextUid: UidType = {
    uid = uid + 1
    uid
  }

  def clear(): Unit = {
    uid = -1
  }
}

trait Node extends {

  /** Reference based equality. */
  override def equals(other: Any): Boolean = other match {
    case ref: AnyRef => this.eq(ref)
    case _ => false
  }

}


trait NodeP[P] extends Node {
  val param: P
  override lazy val hashCode: Int = FactoryP.sHashCode(param)
}

trait NodeNP[K, V <: Node, P] extends Node {
  val kids: Map[K, V]
  val param: P
  override lazy val hashCode: Int = FactoryNP.sHashCode(kids, param)
}

trait NodeListP[K <: Node, P] extends Node {
  val kids: List[K]
  val param: P
  override lazy val hashCode: Int = FactoryListP.sHashCode(kids, param)
}