package util.analyzers

import ast.dsl.InstBoolIdent
import util.io.FileManager

/**
  * Trait for SAT based enumeration of cutsets
  */
trait PhiCutEnumerationIterativeSAT extends PhiCutEnumeration[InstBoolIdent] {
  self: Phi =>

  private val file = FileManager.addFileToTemp("cutEnum")

  /**
    * Compute cutsets with SAT problems resolution
    *
    * @param max cutset cardinality bound
    * @return cutsets
    */
  def computeCutsUntil(max: Int): Iterable[Set[InstBoolIdent]] = {
    if (this.expr.isTrue) {
      Set(Set())
    } else if (this.expr.isFalse) {
      Set()
    } else {
      new IterativeMCSEnumerator(this).apply(max, file)
    }
  }

  /**
    * Compute cutsets with SAT problems resolution
    *
    * @return cutsets
    */
  def computeAllCuts(): Iterable[Set[InstBoolIdent]] = {
    if (this.expr.isTrue) {
      Set(Set())
    } else if (this.expr.isFalse) {
      Set()
    } else {
      new IterativeMCSEnumerator(this).apply(events.size, file)
    }
  }

  override val phiCutEnumerationProfiler = new Profiler[Option[Int], Iterable[Set[InstBoolIdent]]] {
    override def runWithProfile(i: Option[Int])(iterations: Int = 1): (Iterable[Set[InstBoolIdent]], ExecutionTimes) = {
      val (r, preTime) = time({
        val enum = new IterativeMCSEnumerator(self)
        i match {
          case Some(m) =>
            enum.generateFiles(m, file)
          case None =>
            enum.generateFiles(events.size, file)
        }
        enum
      }, iterations)

      val (r2, pTime) = time({
        i match {
          case Some(m) =>
            r.run(m, file)
          case None =>
            r.run(events.size, file)
        }
      }, iterations)

      val (r3, postTime) = time({
        r.parse(r2)
      }, iterations)

      (r3, ExecutionTimes(preTime, pTime, postTime))
    }
  }
}
