package util.analyzers

import ast.dsl.InstBoolIdent
import util.io.FileManager

/**
  * Trait for Pseudo-Boolean based cutsets computation
  */
trait PhiCutEnumerationPB extends PhiCutEnumeration[InstBoolIdent] {
  self: Phi =>
  private val file = FileManager.addFileToTemp("cutEnum")

  /**
    * Compute cutsets with Pseudo-Boolean problems resolution
    *
    * @param max cutset cardinality bound
    * @return cutsets
    */
  def computeCutsUntil(max: Int): Iterable[Set[InstBoolIdent]] = {
    if (this.expr.isTrue) {
      Set(Set())
    } else if (this.expr.isFalse) {
      Set()
    } else {
      new PBMCSEnumerator(this).apply(Some(max), file)
    }
  }

  /**
    * Compute cutsets with Pseudo-Boolean problems resolution
    *
    * @return cutsets
    */
  def computeAllCuts(): Iterable[Set[InstBoolIdent]] = {
    if (this.expr.isTrue) {
      Set(Set())
    } else if (this.expr.isFalse) {
      Set()
    } else {
      new PBMCSEnumerator(this).apply(None, file)
    }
  }

  override val phiCutEnumerationProfiler = new Profiler[Option[Int], Iterable[Set[InstBoolIdent]]] {
    override def runWithProfile(i: Option[Int])(iterations: Int = 1): (Iterable[Set[InstBoolIdent]], ExecutionTimes) = {
      val (r, preTime) = time({
        val enum = new PBMCSEnumerator(self)
        enum.generateFiles(i, file)
        enum
      }, iterations)

      val (r2, pTime) = time({
        r.run(file)
      }, iterations)

      val (r3, postTime) = time({
        r.parse(r2)
      }, iterations)

      (r3, ExecutionTimes(preTime, pTime, postTime))
    }
  }
}
