package util.analyzers

/**
  * Created by kevin on 15/02/17.
  */
case class ExecutionTimes(preProcessTime: Long, processTime: Long, postProcessTime: Long) {
  override def toString: String = {
    s"""
       |setup:${preProcessTime * 1e-9}s
       |compute: ${processTime * 1e-9}s
       |post: ${postProcessTime * 1e-9}s
       |total:${(preProcessTime + processTime + postProcessTime) * 1e-9}s """.stripMargin
  }
}

abstract class Profiler[I, O] {

  def time[R](block: => R, iterations: Int): (R, Long) = {
    val t0 = System.nanoTime()
    for (i <- 1 until iterations) {
      block // call-by-name
    }
    val res = block
    val t1 = System.nanoTime()
    (res, (t1 - t0) / iterations)
  }

  def runWithProfile(i: I)(iterations: Int = 1): (O, ExecutionTimes)
}