package util.analyzers

import java.util.concurrent.{Callable, FutureTask}

import scala.concurrent.ExecutionContext.Implicits.global
import scala.concurrent.duration._
import scala.concurrent.{ExecutionContext, TimeoutException}

/**
  * Created by kdelmas on 31/08/17.
  */

/**
  * Class for timeout task
  * @param toDo the job to do
  * @param timeout the timeout to achieve the job
  * @param onTimeOut the job to perform on timeout
  * @param context the execution context
  * @tparam T the type of returned result
  */
class TimeoutTask[T](toDo: => T, timeout: Duration, onTimeOut: => T)(implicit val context: ExecutionContext) {

  private val t = new FutureTask[T](
    new Callable[T] {
      override def call(): T = toDo
    }
  )

  /**
    * Blocking method to get the result of the execution
    * @return
    */
  def get(): T = {
    context.execute(t)
    try {
      t.get(timeout.toMicros, MICROSECONDS)
    } catch {
      case e: TimeoutException =>
        t.cancel(true)
        if (!t.isDone)
          throw new Exception("thread not stopped")
        onTimeOut
    }
  }
}

object TimeoutTask {
  def apply[T](toDo: => T, timout: Duration, onTimeOut: => T)(implicit context: ExecutionContext): TimeoutTask[T] = new TimeoutTask[T](toDo, timout, onTimeOut)
}
