package util.analyzers

import ast.dsl.InstFlowIdent
import com.microsoft.z3.{BitVecExpr => Z3BitVecExpr, BoolExpr => Z3BoolExpr, Expr => Z3Expr, FuncDecl => Z3FuncDecl, Solver => Z3Solver, Sort => Z3Sort, Symbol => Z3Symbol}
/**
  * compute the SMT expression (exists e1..ek (c in1..inn e1..ek)=(o1..om)) in traceFieldExpr where flows are given as bounded variables
  * referred in traceFieldVars
  */
trait TraceBuild extends ComponentAnalyzer {
  self: Component =>
  val (traceFieldExpr, traceFieldVars) = possibleTraces()

  private def possibleTraces(): (Z3BoolExpr, Map[InstFlowIdent, Z3BitVecExpr]) = {
    val consts: (List[Z3Symbol], List[Z3Sort]) = events.toList.sortBy(p => p._2.getIndex).map(e => (context.mkSymbol(e._1.toString()), context.mkBoolSort())).unzip
    val outVars: Iterable[(InstFlowIdent, Z3BitVecExpr, Z3BitVecExpr)] = outputs.zipWithIndex.map(p => (p._1._1, p._1._2, context.mkBound(inputs.size + events.size + p._2, p._1._2.getSort).asInstanceOf[Z3BitVecExpr]))
    val body: Z3BoolExpr = context.mkAnd(outVars.map(p => context.mkEq(p._2, p._3)).toList: _*)
    (context.mkExists(consts._2.toArray, consts._1.reverse.toArray, body, 0, null, null, null, null), outVars.map(t => (t._1, t._3)).toMap ++ inputs)
  }
}
