package util.io

/**
  * Created by kevin on 14/02/17.
  */

import java.io.Writer

object LatexWriter {
  val newLine = System.lineSeparator()

  def writeTable(data: List[List[String]], columnTitles: List[String], lineTitles: List[String], title: String, to: Writer, label: Option[String] = None): Unit = {
    to.write("""\begin{table}[htb!]""" + newLine)
    to.write("""\centering""" + newLine)
    to.write(s"""\\begin{tabular}{${(0 to columnTitles.size).map(_ => "c").mkString("|")}|}""" + newLine)
    to.write(s"""& ${columnTitles.mkString(" & ")} \\\\""" + newLine)
    to.write("""\hline""" + newLine)
    data.zip(lineTitles).foreach(p => {
      to.write(s"""${p._2} & ${p._1.mkString(" & ")} ${if (p._1.size < columnTitles.size) (0 until (p._1.size - columnTitles.size)).mkString(" & ") else ""} \\\\""" + newLine)
      to.write("""\hline""" + newLine)
    })
    to.write("""\end{tabular}""" + newLine)
    to.write(s"""\\caption{$title}""" + newLine)
    for (l <- label) yield to.write(s"""\\label{$l}""" + newLine)
    to.write(s"""\\end{table}""" + newLine)
    to.flush()
  }

  case class RowDSEPerf(method: String,
                        orderReq: Int,
                        UnrReq: Double,
                        Rosace: String = "not done",
                        Fuel: String = "not done",
                        Quad: String = "not done",
                        HBS: String = "not done") {
    def write(to: Writer) = {
      to.write(s"""$method & $orderReq & $UnrReq & $Rosace & $Fuel & $Quad & $HBS \\\\""" + newLine)
    }
  }

  def writePerDSE(title: String, rows: List[RowDSEPerf], to: Writer): Unit = {
    to.write("""\begin{table}[htb!]""" + newLine)
    to.write("""\centering""" + newLine)
    to.write(s"""\\begin{tabular}{ccccccc}""" + newLine)
    to.write("""\toprule""" + newLine)
    to.write("""&&& \multicolumn{4}{c}{System}\\""" + newLine)
    to.write("""\cmidrule{4-7}""" + newLine)
    to.write("""Method & Order & R & Rosace & Fuel & Quadcopter & HBS\\""" + newLine)
    to.write("""\midrule""" + newLine)
    rows.foreach(row => row.write(to))
    to.write("""\bottomrule""" + newLine)
    to.write("""\end{tabular}""" + newLine)
    to.write(s"""\\caption{$title}""" + newLine)
    to.write("""\end{table}""" + newLine)
    to.flush()
  }

  case class RowCutPerf(System: String,
                        k: Int,
                        HipHop: String = "not done",
                        xSAP: String = "not done",
                        XFTA: String = "not done",
                        KCR: String = "not done") {
    def write(to: Writer) = {
      to.write(s"""$System & $k & $HipHop & $xSAP & $XFTA & $KCR \\\\""" + newLine)
    }
  }

  def writePerCut(title: String, rows: List[RowCutPerf], to: Writer): Unit = {
    to.write("""\begin{table}[htb!]""" + newLine)
    to.write("""\centering""" + newLine)
    to.write(s"""\\begin{tabular}{cccccc}""" + newLine)
    to.write("""\toprule""" + newLine)
    to.write("""&&& \multicolumn{4}{c}{Tool}\\""" + newLine)
    to.write("""\cmidrule{3-6}""" + newLine)
    to.write(""" System & k & HipHOPS & XSAP & XFTA & KCR Analyser\\""" + newLine)
    to.write("""\midrule""" + newLine)
    rows.foreach(row => row.write(to))
    to.write("""\bottomrule""" + newLine)
    to.write("""\end{tabular}""" + newLine)
    to.write(s"""\\caption{$title}""" + newLine)
    to.write("""\end{table}""" + newLine)
    to.flush()
  }
}
