package util.io

import java.io.{BufferedWriter, FileWriter, IOException}

/**
  * Created by rdelmas on 21/05/16.
  */
case class WriteToFile(fileName: String, reThrow: Boolean = false) extends ((BufferedWriter => Unit) => Unit) {

  def apply(writeToFile: (BufferedWriter => Unit)): Unit = {
    var out: BufferedWriter = null
    try {
      out = new BufferedWriter(new FileWriter(fileName))
      writeToFile(out)
    } catch {
      case e: IOException => {
        if (reThrow) {
          throw e
        } else {
          println(s"Could not create file ${fileName} for writing: ${e.toString}")
          sys.exit(1)
        }
      }
      case e: Throwable => throw e
    } finally {
      if (out != null)
        out.close()
    }
  }

}
