package util.sat

import scala.collection.immutable


/**
  * Created by rdelmas on 20/06/16.
  */

/** An interface for building optimization problems on top of CNF formulas. */
trait OptimizeLike {
  this: CnfLike =>

  /** Abstract trait for optimization criteria. */
  sealed trait Criterion {
    val lits: List[Int]
    val weights: List[Int]
    val k: Int

    /** Converts any minimize or maximize criterion to an equivalent Minimize critierion. */
    def toMinimize: Minimize
  }

  /** A linear criterion to be maximized. */
  case class Maximize(lits: List[Int], weights: List[Int], k: Int = 0) extends Criterion {
    def toMinimize = Minimize(lits, weights.map(w => -w), -k)
  }

  /** A linear criterion to be minimized. */
  case class Minimize(lits: List[Int], weights: List[Int], k: Int = 0) extends Criterion {
    def toMinimize = this
  }

  /** List of optimization criteria. Lexicographic optimization if there is more than one criterion. */
  private var _criteria: immutable.Vector[Criterion] = immutable.Vector()

  /** Adds an optimization criterion to the formula. */
  final def addCriterion(c: Criterion): Unit = this._criteria :+= c

  /**
    * @return the number of criteria of the formula.
    **/
  final def nofCriteria: Int = this._criteria.length

  /** @return An iterator on criteria of the formula. */
  final def criteria: Iterator[Criterion] = this._criteria.iterator

}
