package util.sat

/**
  * Created by rdelmas on 27/06/16.
  *
  * Encodes a Prop formula to a FormulaLike (CNF + Circuit) with the special dual-rail ternary logic encoding.
  *
  */
object PropToTernaryEncoder extends (Prop.Expr => (FormulaLike, PartialFunction[Prop.Atom, (Int, Int)], PartialFunction[Prop.Expr, (Int, Int)])) {

  def apply(formula: Prop.Expr): (FormulaLike, PartialFunction[Prop.Atom, (Int, Int)], PartialFunction[Prop.Expr, (Int, Int)]) = {
    val toNNF = new PropToNNF
    val nnf = toNNF(formula)
    val circuit = new FormulaLike {}
    val primaryEncoder = NNFToCircuitEncoder(nnf, circuit, switchableGates = true)
    val dualRail: FormulaLike = new FormulaLike {}
    val dualRailEncoder = new DualRailEncoder(circuit, dualRail)
    val switchEncoder = SwitchEncoder(nnf, circuit, dualRail, dualRailEncoder)
    primaryEncoder.encode()
    dualRailEncoder.encode()
    switchEncoder.encode()
    val atomFunc = toNNF.atomCacheFunc andThen primaryEncoder.atomCacheFunc andThen dualRailEncoder.funcMap
    val exprFunc = toNNF.exprCacheFunc andThen primaryEncoder.exprCacheFunc andThen dualRailEncoder.funcMap
    (dualRail, atomFunc, exprFunc)
  }

}
