/** $Id$
  *
  * @author $Author$
  * @date $Date$
  **/
package util.stack

abstract trait SolverProxy[BoolExpr] {
  /** Name of the solver (used in debug prints). */
  def name: String

  /** Adds an assertion to the current top level assertion set. */
  def add(a: BoolExpr): Unit

  /** Adds a list of assertions to the current top level assertion set. */
  def add(as: List[BoolExpr]): Unit

  /** Pops all the stack and restores the solvers initial state. */
  def reset(): Unit
}

/** Abstract type for a proxy to a statefull solver with an assertion stack. */
abstract trait SolverProxyStackMethods[BoolExpr] {
  this: SolverProxy[BoolExpr] =>
  /** Pushes a new empty assertion set on the assertion stack. */
  def push(): Unit

  /** Pops the topmost n levels of the assertion stack. 'n' must be < stacksize-1. */
  def pop(n: Int = 1): Unit
}

/** Abstract type for a proxy to a statefull solver that allows to create fresh variables, negations, conjunctions, disjunctions. */
abstract trait SolverProxyMakeMethods[BoolIdent <: BoolExpr, BoolExpr] {
  this: SolverProxy[BoolExpr] =>
  /** Creates a fresh variable. */
  def mkFresh: BoolIdent

  /** Creates a negation. */
  def mkNot(l: BoolExpr): BoolExpr

  /** Creates a conjuction. */
  def mkAnd(l: BoolExpr, r: BoolExpr): BoolExpr

  /** Creates a disjunction. */
  def mkOr(l: BoolExpr, r: BoolExpr): BoolExpr
}


abstract trait StackSolverProxy[BoolExpr] extends SolverProxy[BoolExpr] with SolverProxyStackMethods[BoolExpr]
