package util.translators

/**
  * Created by kevin on 16/02/17.
  */

import scala.xml._

object XmlHelpers {

  implicit object XMLHelperImplicits {
    implicit def nodeExtras(n: Node): NodeExtras = new NodeExtras(n)

    implicit def elemExtras(e: Elem): ElemExtras = new ElemExtras(e)
  }

  val docBuilder =
    javax.xml.parsers.DocumentBuilderFactory.newInstance().newDocumentBuilder()
}


class NodeExtras(n: Node) {

  import XmlHelpers.XMLHelperImplicits._

  def toJdkNode(doc: org.w3c.dom.Document): org.w3c.dom.Node =
    n match {
      case Elem(prefix, label, attributes, scope, children@_*) =>
        // XXX: ns
        val r = doc.createElement(label)
        for (a <- attributes) {
          r.setAttribute(a.key, a.value.text)
        }
        for (c <- children) {
          r.appendChild(c.toJdkNode(doc))
        }
        r
      case Text(text) => doc.createTextNode(text)
      case Comment(comment) => doc.createComment(comment)
      // not sure
      case a: Atom[_] => doc.createTextNode(a.data.toString)
      // XXX: other types
      //case x => throw new Exception(x.getClass.getName)
    }
}

class ElemExtras(e: Elem) extends NodeExtras(e) {
  override def toJdkNode(doc: org.w3c.dom.Document) =
    super.toJdkNode(doc).asInstanceOf[org.w3c.dom.Element]

  def toJdkDoc = {
    val doc = XmlHelpers.docBuilder.newDocument()
    doc.appendChild(toJdkNode(doc))
    doc
  }
}


